import idlsave as idl
import numpy as np
import pyfits as py
import directories

import ipdb as pdb


def acs_model_e( x, y, wavelength='814', data_dir=None):
    '''
    ;+
    ; NAME:
    ;       ACS_MODEL_E
    ;
    ; PURPOSE:
    ;       Obtain the ellipticity of a TinyTim model PSF at one
    ;       position and any focus value(s) on ACS.
    ;  
    ; CATEGORY:
    ;       ACS data reduction.
    ;
    ; CALLING SEQUENCE:
    ;       result=acs_model_e(x, y, focus, catalogues=catalogues)
    ;
    ; INPUTS:
    ;       X          - Object x coordinate.
    ;       Y          - Object y coordinate.
    ;
    ; OPTIONAL INPUTS:
    ;       FOCUS      - Focus offset from optimal [um].
    ;                    Can be one number or an array.
    ;       CATALOGUES - Array of model ellipticities. If not set (e.g. on first
    ;                    use), will read in from disk. If already set, will recall
    ;                    from memory, for speed.
    ;       DITHER     - Dither pattern (PSF is averaged from exposures at all of
    ;                    these positions)
    ;
    ; KEYWORD PARAMETERS:
    ;       None.
    ;
    ; OUTPUTS:
    ;       Returns a structure containing model ellipticities and other parameters.
    ;
    ; OPTIONAL OUTPUTS:
    ;       None.
    ;
    ; COMMON BLOCKS:
    ;       None.
    ;
    ; SIDE EFFECTS:
    ;       None.
    ;
    ; RESTRICTIONS:
    ;       None.
    ;
    ; EXAMPLE:
    ;       See use in acs_determine_focus.pro.
    ;
    ; MODIFICATION HISTORY:
    ;       Feb 05 - All shape moments needed by RRG now returned. RM.
    ;       Feb 05 - Written by Richard Massey.
    ;-
    ; 
  '''

    dirs = directories.return_dirs()


    # Initialise variables
  

    filenames1=dirs.psf_model_dir+"/TinyTim_f"
    filenames2=".moms"
    n_focus = 16
    catalogues_focus = np.arange( n_focus ) - 10
    catalogues_string = np.array([ "f"+str(np.int(i)) for i in catalogues_focus])
    n_coords=len(x)
  
    # Load data (required on first use only)
    catalogues={ 'focus':catalogues_focus,
                 'focus_string':catalogues_string}
        
  
    # Prepare empty arrays to contain the answer
    answer = return_moms( n_focus, n_coords, x, y)
    
    # Loop over each focus value in turn
    for f in xrange(n_focus):
        moms = idl.read( filenames1+str(catalogues_focus[f])+filenames2).moms
        
        
        # Loop over each position in which we're interested
        for i in xrange(n_coords):
            # Find closest model star (i.e. don't inerpolate)
            rmin = np.sqrt( (moms.x[0]-x[i])**2 + (moms.y[0]-y[i])**2)
            match = rmin == np.min(rmin)
            
            # Store its ellipticity and shape moments

            answer.e1[f,i]=moms.e1[0][match]
            answer.e2[f,i]=moms.e2[0][match]
            answer.xx[f,i]=moms.xx[0][match]
            answer.xy[f,i]=moms.xy[0][match]
            answer.yy[f,i]=moms.yy[0][match]
            answer.xxxx[f,i]=moms.xxxx[0][match]
            answer.xxxy[f,i]=moms.xxxy[0][match]
            answer.xxyy[f,i]=moms.xxyy[0][match]
            answer.xyyy[f,i]=moms.xyyy[0][match]
            answer.yyyy[f,i]=moms.yyyy[0][match]
      
            #Store its other quantities
            answer.x_model[f,i]=moms.x[0][match]
            answer.y_model[f,i]=moms.y[0][match]
            answer.offset_model[f,i]=np.min(rmin)
  
 

    return answer, catalogues_focus

class return_moms( dict ):

    def __init__( self, n_focus, n_coords, x, y):

        self.__dict__['x'] = x
        self.__dict__['y'] = y
        self.__dict__['e1']=np.zeros((n_focus,n_coords))
        self.__dict__['e2']=np.zeros((n_focus,n_coords))
        self.__dict__['xx']=np.zeros((n_focus,n_coords))
        self.__dict__['xy']=np.zeros((n_focus,n_coords))
        self.__dict__['yy']=np.zeros((n_focus,n_coords))
        self.__dict__['xxxx']=np.zeros((n_focus,n_coords))
        self.__dict__['xxxy']=np.zeros((n_focus,n_coords))
        self.__dict__['xxyy']=np.zeros((n_focus,n_coords))
        self.__dict__['xyyy']=np.zeros((n_focus,n_coords))
        self.__dict__['yyyy']=np.zeros((n_focus,n_coords))
        self.__dict__['x_model']=np.zeros((n_focus,n_coords))
        self.__dict__['y_model']=np.zeros((n_focus,n_coords))
        self.__dict__['offset_model']=np.zeros((n_focus,n_coords))

    def keys(self):
        return self.__dict__.keys()

    def __getitem__(self, key): 
        return self.__dict__[key]

            



import numpy as np

def combination( n, k):

    '''
    ;PURPOSE : TO FIND THE BINOMIAL COEEFICIENT OF SET OF N 
    ;          ELEMENTS AND K COMINATIONS
    
    ;INPUTS : 
    ;     N : the number in the set
    ;     K : the number of combinations
    
    
    ;RETURNS : 
    ;    ( N )  =     N!
    ;    ( K )     ------
    ;              K!(N-K)!
    '''
    
    return np.math.factorial(n)/(np.math.factorial(k)*np.math.factorial(n-k))




import numpy as np
import ipdb as pdb
import pyfits as py

def calc_shear( corrected_moments, galaxies, outfile,
                min_rad=6, mult=2,
                size_cut=[0., 100.],
                mag_cut=[22.5, 30],
                signal_noise_cut=3.,
                rhodes_factor=0.86,
                dataDir='./',
                expThresh=4,
                stat_type='median'):
    '''
    PURPOSE : TO WORK OUT VARIOUS FACTORS AND ADJUST THE PSF
    CORRECTED ELLIPICITY FOR THESE AND GET AN ESTAIMTE OF THE SHEAR


    INPUT  :
        CORRECTED_MOMENTS : A fitsrec structure of the corrected
                            and uncorrected moments to measure
                            and estimate the shear.
        GALAXIES : A integer vector of indexes that correspond to those
                   CORRECTED_MOMENTS that have been classified as a galaxy
        OUTFILE : The name of the fits file in which the shears, gamma1 and gamma2
                have been estimated as


    OPTIONAL INPUTS:
         MIN_RAD: The minimum measured radios that is used to weight each galaxies in the
                  measurement.
        SIZE_CUT [LO,HI] : scalar, the cut in galaxy size that is allowed to measure shear on
        MAG_CUT [LO,HI] : two scalar array, the cut in the galaxy magnitude that is allowed to measure shear on
        SIGNAL_NOSIE_CUT : scalar, the lowest acceptable signal to nosie allowed in the catalogue
        rhodes_factor : The RRG rhodes factor (see paper)
        dataDir : the directory in which all the data exists
        expThresh : the minimum number of exposures a galaxy has to have an acceptably robust shape.
        stat_type :  the stat type used over the field of galaxies to estimate the corrections to
                     from elliptcitiy to shear.
        
    OUTPUT :
         OUTFILE : A fitsfile named outfile with the same fields as corrected_moments, except with
                   two extra fields. GAMMA1 and GAMMA2, the two componenets of the estimated shear.

    '''
    #Need to filter as i determine mean quantities that
    #shoudlnt be used from bad galaxies
    signal_noise = corrected_moments['FLUX_AUTO'] / \
      corrected_moments['FLUXERR_AUTO']
  
    uncut_ell_sqr = corrected_moments['e1']*corrected_moments['e1'] + \
          corrected_moments['e2']*corrected_moments['e2']

    uncor_size = np.sqrt( 0.5*(corrected_moments.xx_uncorrected + \
                               corrected_moments.yy_uncorrected))
          
    good = np.zeros(len(corrected_moments.x))
    good[ galaxies ] = 1
    good[ (corrected_moments.xx + corrected_moments.yy < 0)] = 0
    good[ (uncut_ell_sqr > 2 ) ]  = 0
    good[ (uncor_size < size_cut[0] )]  = 0 
    good[ (uncor_size > size_cut[1] )]  = 0
    good[( corrected_moments.MAG_AUTO < mag_cut[0] )]  = 0 
    good[( corrected_moments.MAG_AUTO > mag_cut[1] )]  = 0
    good[ (signal_noise < signal_noise_cut)]  = 0 
    
    #good[ corrected_moments.nExposures < expThresh ] = 0
    good[  (~np.isfinite(corrected_moments.xx)) ] = 0
    good[  corrected_moments.prob != 0 ] = 0

    
    momc = corrected_moments[good == 1]
    size = np.sqrt( 0.5*(momc.xx + momc.yy))

    nObjects=len(momc.xx)

    weight = momc['radius']
    weight[ momc['radius'] < min_rad] = min_rad
    
    beta = 1./(2.*momc['gal_size']**2*(momc['shear']**2+weight**2))
    
    u1 = beta*(-momc['xxxx']+momc['yyyy']) 
    u2 = -2.*beta*(momc['xxxy']+momc['xyyy'])
    
    gal_lambda=beta*(momc['xxxx']+2.*momc['xxyy']+momc['yyyy'])
  
    
    ellipticity_sqr = momc['e1']**2+momc['e2']**2
    e_dot_u = momc['e1']*u1+momc['e2']*u2
    e_cross_u = momc['e1']*u2-momc['e2']*u1

    if stat_type == 'mean':
        #These are the mean G1, G2
        G2 = 0.5*np.nanmean(e_cross_u)
        G1 = 2-np.nanmean(ellipticity_sqr)-\
            0.5*np.nanmean(gal_lambda)-\
            0.5*np.nanmean(e_dot_u)
    elif stat_type =='median':
        #The median
        G2 = 0.5*np.nanmedian(e_cross_u)
        
        G1 = 2-np.nanmedian(ellipticity_sqr)- 0.5*np.nanmedian(gal_lambda)-0.5*np.nanmedian(e_dot_u)
    else:
        raise ValueError("Stat type not recognised")
    
    gamma1=momc['e1']/G1/rhodes_factor
    gamma2=momc['e2']/G1/rhodes_factor

    fits_cols = []
    for iName in momc.columns.names:
        fits_cols.append( py.Column(name=iName, format=momc[iName].dtype, array=momc[iName] ) )
        
    newcol = [ py.Column(name='gamma1', format=gamma1.dtype, array=gamma1),
               py.Column(name='gamma2', format=gamma2.dtype, array=gamma2) ]

 
    hdu = py.BinTableHDU.from_columns(fits_cols + newcol)
    hdu.writeto(outfile, clobber=True)




import numpy as np
import ipdb as pdb

def acs_map_xy( x, y,
                inverse=False,        
                order=np.inf,            
                chip=None,              
                pixel_scale=0.05):

    '''

    ;+
    ; NAME:
    ;       ACS_MAP_XY
    ;
    ; PURPOSE:
    ;       Provides the coordinate tranformation to map between 
    ;       positions on raw ACS images, and those after correction
    ;       for astrometic distortions within the ACS camera.
    ;       Such a correction is applied by multidrizzle.
    ;
    ;       Note that only the DRIZZLE polynomial is calculated; the
    ;       distortion map adjusts positions by a further ~0.1 pixels.
    ;       This needs to be applied separately, but if the inverse
    ;       transformation is needed, at least an addition is easy to
    ;       invert!
    ;  
    ; CATEGORY:
    ;       ACS data reduction.
    ;
    ; CALLING SEQUENCE:
    ;       result=acs_map_xy(x_coords,y_coords)
    ;
    ; INPUTS:
    ;       X           - Object x coordinate, or an array of them.
    ;       Y           - Object y coordinate, or an array of them.
    ;
    ; OPTIONAL INPUTS:
    ;       PIXEL_SCALE - The output pixel scale as used by DRIZZLE.
    ;       OFFSET      - The [x,y] position of the bottom-left pixel after
    ;                     correction for geometric distortions. If an image
    ;                     consists of multiple, dithered exposures, this
    ;                     should vary for each exposure, reflecting the
    ;                     overall dither pattern.
    ;       CHIP        - Which CCD the objects are on. If omitted, it is
    ;                     assumed that the objects are all on a combined 
    ;                     4096x4096 image, with no gap between the CCDs.
    ;                     Chip #1 is taken to be the bottom one.
    ;       ORDER       - Maximum order of polynomial used for the 
    ;                     transformation. Default is set by the number of read
    ;                     ceofficients (which gives a quartic).
    ;
    ; KEYWORD PARAMETERS:
    ;       INVERSE     - Apply the inverse transformation, from the frame
    ;                     after correction to the raw CCD frame.
    ;
    ; OUTPUTS:
    ;       Returns a structure containing transformed x' and y' coords.
    ;
    ; OPTIONAL OUTPUTS:
    ;       None.
    ;
    ; COMMON BLOCKS:
    ;       None.
    ;
    ; SIDE EFFECTS:
    ;       None.
    ;
    ; RESTRICTIONS:
    ;       None.
    ;
    ; EXAMPLE:
    ;       A full demonstration is provided in acs_map_xy_demo.pro.
    ;
    ; MODIFICATION HISTORY:
    ;       Nov 05 - Memory use with large arrays streamlined by RM.
    ;       Nov 05 - Chips distinguished with different pixels scales by RM.
    ;       Aug 05 - PIXEL_SCALE keyword added by RM.
    ;       Jan 05 - Inverse distortion coefficients supplied by Herve Aussel.
    ;       Jan 05 - Written by Richard Massey.
    ;-
    '''
    
    # Desired position of bottom-left pixel after mapping, to enable dithering
    # (STScI standard notation seems to use the centre of the chip instead).

    if pixel_scale == 0.05:
        offset=np.array([199,210])
    elif pixel_scale == 0.03:
        offset=np.array([247,224])
    else:
      offset=np.array([0.,0.])
      print "Default offset not known for this pixel scale! Assuming zero."
      
    

    n_positions=len(x)
    if len(y) != n_positions:
        raise ValueError( "x and y arrays are of different sizes!")


  # Set polynomial coefficients for transformation (or read in using read_coeffs.pro).
    if not inverse:
        # Centre of the two chips.
        origin1=np.array([2048.,1024.])
        origin2=np.array([2048.,3072.])
        # Coefficients obtained from *_flt_coeffs2.dat, output by DRIZZLE
        chip1xcoeffs=np.array([99.565188, 4.9825169E-02, 1.9619130E-03,
                            4.2225790E-07,-2.4670911E-07, 9.3607483E-08,
                            -2.3507861E-11, -3.2000680E-12, -2.6439229E-11,
                            -2.9675790E-13, 1.2487490E-15, 2.4934469E-16,
                            1.8191949E-15,-9.3522379E-16,  7.0261870E-16])/5E-2
        
        chip1ycoeffs=np.array([50.887074, 1.4886410E-03, 5.0269939E-02,
                            -7.4966152E-08, 3.0282621E-07, -3.6023110E-07,
                            3.6018710E-12, -2.5871029E-11, -4.1843920E-12,
                            -2.1149299E-11, -8.7282642E-16, -9.6736218E-17,
                            -1.4343210E-15, 6.6963921E-16,  5.3896481E-17])/5E-2
        
        chip2xcoeffs= np.array([103.83470, 4.9219720E-02, 2.2672340E-03,
                                4.1262069E-07, -3.5540299E-07, 8.8666752E-08,
                                -2.2995350E-11, -6.2186931E-12, -2.6492589E-11,
                                2.0806501E-12, 8.6975979E-16, 1.6196320E-15,
                                4.6171330E-15,-1.0479930E-15,  6.8707738E-15])/5E-2
        
        chip2ycoeffs=np.array([154.76950, 2.1297960E-03, 4.8578139E-02, -1.2579410E-07,
                            2.9587099E-07, -4.6947380E-07, 3.4025090E-12,
                            -2.1720251E-11, -2.6551810E-12, -1.9880131E-11,
                            -8.3371838E-16, -1.7790191E-15, -5.3674559E-15,
                            1.3629340E-15, -8.4030801E-15])/5E-2
    else:
        # Centre of the two chips after forward transformation.
        origin1=np.array([2047.9090,1089.8271])+offset
        origin2=np.array([2133.2993,3167.4756])+offset
        # Reverse transformation coefficients, calculated using Herve Aussel's fit_polynomial_distortion.pro.
        chip2xcoeffs=np.array([2048+17.690567, 1.0098704, -0.041546981,
                               -9.0331744e-06, 7.5455644e-06, -3.5491556e-06,
                               6.5557962e-10, -1.8975432e-11, 5.0821254e-10,
                               6.3986712e-10, -3.5403050e-14, 1.2697593e-14,
                               -1.4901954e-13, 1.0700322e-13, -1.9272843e-13])
        
        chip2ycoeffs=np.array([4096-1067.8955, -0.036814084, 1.0101043,
                               2.8356072e-06, -7.4504696e-06, 1.0394685e-05,
                               -1.9416512e-10, 3.4142281e-10, -7.1805489e-11,
                               -4.5094846e-10, 2.4085682e-14, -2.2323542e-15,
                               1.8504679e-13, -1.1194485e-13, 2.6685584e-13])
        
        chip1xcoeffs=np.array([2048+14.317421, 1.0106205, -0.043983312,
                               -8.9777910e-06, 7.4831681e-06, -2.6797870e-06,
                               6.3869379e-10, -3.7342433e-10, 7.9598247e-10,
                               -1.5421049e-10, -3.9667949e-14, 2.1662802e-14,
                               -6.4200504e-14, 4.3270883e-14, -1.8717430e-14])
        
        chip1ycoeffs=np.array([1032.3554, -0.036663688, 1.0124915,
                               2.6735703e-06, -7.1984241e-06, 9.1768566e-06,
                               -1.6076053e-10, 7.8555927e-10, -3.0590836e-10,
                               6.1370412e-10, 2.1460207e-14, -2.1748604e-14,
                               5.9100563e-14, -3.2919786e-14, 1.9975680e-14])
        

    order=np.int(np.min([order,np.round((np.sqrt(9-8*(1-len(chip1xcoeffs)))-3)/2.)])) # i.e. quartic!

    # Decide which chip the objects are on.
    if  chip is None:      
        # Objects are on some combined image (assumed to be 4096x4096, with no gap).
        if inverse:
            # Work out which is on which CCD, by first getting the distorted CCD boundary
            boundary=acs_map_xy([0,4095,0,4095],[2047,2047,2048,2048],
                                order=order,
                                pixel_scale=pixel_scale,
                                offset=offset)
            boundary_x1=np.mean([boundary.x[0],boundary.x[2]])
            boundary_x2=np.mean([boundary.x[1],boundary.x[3]])
            boundary_y1=np.mean([boundary.y[0],boundary.y[2]])
            boundary_y2=np.mean([boundary.y[1],boundary.y[3]])
            boundary_m=(boundary_y2-boundary_y1)/(boundary_x2-boundary_x1)
            boundary_c=boundary_y1-boundary_m*boundary_x1
            
            chip1= np.arange(len(y))[y < boundary_m*x+boundary_c]
            chip2= np.arange(len(y))[y >= boundary_m*x+boundary_c]
            
            n_chip1 = len(chip1)
            n_chip2 = len(chip2)

        else:
            # Boundary is a horizontal line between pixels 2047 and 2048
            chip1= np.arange(len(y))[y < 2048.]
            chip2= np.arange(len(y))[y >= 2048.]
            n_chip1 = len(chip1)
            n_chip2 = len(chip2)
            
        chipid=np.ones(len(x))
        if n_chip2 > 0:
            chipid[chip2]=2.
    else:
        # All objects are on one of the chips, specified by the user.
        if inverse:
            raise ValueError("Not yet implemented this option!")
        if len(chip) > 1:
            raise ValueError("Not yet implemented this option!")
        if chip == 1:
            n_chip1=len(x)
            chip1=np.arange(n_chip1)
            n_chip2=0
        
        elif chip == 2:
            n_chip2=len(x)
            chip2=np.arange(n_chip2)
            n_chip1=0
            origin2[1] -= 2048
        
        else:
            raise ValueError("ACS only has two CCDs!")
  
        chipid=np.zeros(len(x))+chip



    # Create some empty arrays of the right size, in which to put the final answer.
    xprime=np.zeros(len(x))
    yprime=np.zeros(len(y))


    # Perform transformation via the polynomial model.
    # THIS IS BLOODY SLOW IF THERE ARE A LOT OF POSITIONS!
    print n_chip1
    if n_chip1 > 0 :
        k=0
        for i in xrange(order+1):
            for j in xrange(i+1):
                xprime[chip1] += chip1xcoeffs[k]*(x[chip1]-origin1[0])**(i-j)*(y[chip1]-origin1[1])**(j)
                yprime[chip1] += chip1ycoeffs[k]*(x[chip1]-origin1[0])**(i-j)*(y[chip1]-origin1[1])**(j)
         
                k+=1

    if n_chip2 > 0:
        k=0
        for i in xrange(order+1):
            for j in xrange(i+1):
                xprime[chip2] += chip2xcoeffs[k]*(x[chip2]-origin2[0])**(i-j)*(y[chip2]-origin2[1])**(j)
                yprime[chip2] += chip2ycoeffs[k]*(x[chip2]-origin2[0])**(i-j)*(y[chip2]-origin2[1])**(j)
                k+=1
        
  






    # Shift to dithered position (polynomial fit has a strange convention to put bottom-
    # left pixel at (-56.6,72.1) by default. Perhaps some other place stays invariant).
    if not inverse:
        xprime = (xprime+56.605274)*0.05/pixel_scale+offset[0]
        yprime = (yprime+72.085616)*0.05/pixel_scale+offset[1]
    else:
        xprime *= pixel_scale/0.05  #-56.605274
        yprime *= pixel_scale/0.05  #-72.085616



    # Tell the world what we have found!
    new_coeffs=coeffs( chipid, xprime, yprime )
    
    return new_coeffs


class coeffs:

    def __init__( self, chip, x ,y):
        self.chip = chip
        self.x = x
        self.y = y



import os as os
import numpy as np
import glob as glob
import RRGtools as at
import pickle as pkl
import measure_moms as measure_moms
import star_galaxy_separation as sgs
import pyfits as py
import calc_shear as cs
import psf_cor as psf
import ipdb as pdb
import plot_shears as plot
import ellipse_to_reg as etr
import directories as directories
import rrg_to_lenstool as rtl
import subprocess

def main(  infile, hst_filter=None,
            data_dir=None,
            code_dir=None,
            sex_files=None,
            psf_model_dir=None,
            expThresh = 3, 
            noisy=False, 
            nonstop=True, 
            fits_cat=None, 
            mag_cut=[0.,40.], 
            signal_noise_cut=4.4,
            size_cut=[3., 30.],
            min_rad=6.,
            mult=2.):
    '''
    ;PURPOSE : RUN RRG OVER THE GIVEN CLUSTER AND FILTER, CAN TAKE
    ;          IN MULTIPLE EXPOSURES
    ;          
    ;          THEN FILTERS THE CATALOGUE AND USES ONLY THOSE
    ;          GALAXIES WHICH HAVE EXPTHRESH NUMBER OF EXPOSURES
    ;          OVER THEM AND OUTPUTS .RRG FILE WHICH IS THE CAT
    ;          IN X-Y COORDINATESD
    
    ;          AND THEN RUNS ELLCONVERTER WHICH INVOKES THE MASKS
    ;          IN THE DS9 FILE, MASK.REG AND OUTPUTS A .LENSTOOL
    ;          FILE WHICH IS READY TO BE PUT ITO LENSTOOL
    
    
    ;INPUTS : CLUSTER_NAME : THE NAME OF THE CLUSTER
    ;         FILTER : THE HST FILTER USED
    
    ;KEYWORDS : EXPTHRESH : THE MINIMUM NUMBER OF EXPOSURES ONE
    ;                       GALAXY MUST HAVE BEFORE ALLOWED TO
    ;                       BE IN SAMPLE; DEFAULT = 2 TO PREVENT
    ;                       EDGE GALAXIES IN THE DITHER TO BE INCL.
    ;           FILTER2 : THE SECOND FILTER USED TO GET THE RED_SEQUENCE
    ;                     IF NOT SET THEN RED_SEQUENCE IS NOT FOUND
    ;           NOISY : RUN THE SOURCE EXTRACTION USING THE OLD AND
    ;                   NOT THE MATHILDE NEW ONE THAT IS MROE SENSITIVE
    ;           NONSTOP : DO NOT PAUSE FOR CONFIRATION I WANT TO CONTINUE
    ;           FITS_CAT :  MAKE A CATALOGUE THAT IS IN FITS FORMAT
    ;           MAG_CUT : MAGNITUDE CUTS
    

    ;TO DO : 
    ;        2. CHANGE SUCH THAT FUNCTIONS DONT CONSISTENTLY READ
    ;           FITS IMAGES AND SLOW THIGNS DOWN
    
    '''
    if hst_filter is None:
        hst_filter='F814W'
    wavelength=''.join([  s for s in hst_filter if s.isdigit()])
                   
    #SET GLOBAL PARAMETERS TO BE USED FOR ALL
    #get the cwd first, and make sure these are aboslute paths!
    if code_dir is None:
        code_dir = '/'.join(os.path.abspath(__file__).split('/')[:-1])
        
    if sex_files is None:
        sex_files=code_dir+'/sex_files/'
    
    if  psf_model_dir is None:
        psf_model_dir=code_dir+'/psf_lib/'
   
    
    if data_dir is None:
        data_dir = os.getcwd()+'/'    
   
    stilts_dir = '/'.join(subprocess.check_output(['which','stilts.sh']).split('/')[:-1])
        
    dirs = directories.directories(data_dir,  sex_files,
                           psf_model_dir+'/'+str(wavelength)+'/',
                                       code_dir, stilts_dir)
    dirs.check_dirs()
    dirs.write_dirs()

    #Check files exist
    field=dirs.data_dir+infile
    if not os.path.isfile( field ):
        raise ValueError('Cant find input image (%s)' % field)

    if not os.path.isfile( field):
        raise ValueError("%s not found" % field)
  
    Exposures = glob.glob( dirs.data_dir+'/j*.fits ')
    nExposures = len(Exposures)
  
    if nExposures  < expThresh:
        expThresh = nExposures
        print 'WARNING: Low number of exposures'
    
  

    # Define survey parameters
    #------------------------------------------
    #Now as keywords

 
    sex_catalogue = field[:-5]+"_sex.cat"
    
    #Find objects and measure their raw shapes
    if not os.path.isfile( sex_catalogue):
        weight_file = infile[:-8]+'wht.fits'
        sources = at.source_extract( infile, weight_file,
                                         outfile=sex_catalogue,
                                         conf_path=dirs.sex_files,
                                         dataDir=dirs.data_dir)
    else:
        sources = py.open( sex_catalogue )[1].data

     
        
    print sex_catalogue
  
  
    uncorrected_moments_cat = field[:-5]+"_uncor.cat"
    
    if not os.path.isfile(uncorrected_moments_cat):
        measure_moms.measure_moms( infile,
                                   sex_catalogue,
                                   uncorrected_moments_cat,
                                    min_rad=min_rad, mult=mult)

    uncorrected_moments = py.open( uncorrected_moments_cat )[1].data
 

    
    galaxies, stars = sgs.star_galaxy_separation( uncorrected_moments,
                                                  savefile='galStar.locus' )

    
    n_stars=len(stars)
  
    corrected_moments_cat = field[:-5]+"_cor.cat"

    #Correct for the PSF
    if not os.path.isfile(corrected_moments_cat):
         psf.psf_cor( uncorrected_moments_cat,
                    corrected_moments_cat,
                    infile, wavelength,
                    mult=1, min_rad=min_rad, chip=1,
                    constantpsf=0, mscale=0, 
                    num_exposures=1, order=3,
                    n_chip=2)
    

    corrected_moments = py.open( corrected_moments_cat )[1].data

    #Correct zerpoint for the stacked num exposures
  
    sheared_cat = field[:-5]+".shears"
    
    cs.calc_shear( corrected_moments, galaxies,
                   sheared_cat, 
                    min_rad=min_rad, mult=mult,
                    signal_noise_cut=signal_noise_cut,
                    size_cut=size_cut,
                    mag_cut=mag_cut,
                    dataDir=data_dir)
    

    
    plot.plot_shears( sheared_cat )
    etr.ellipse_to_reg( sheared_cat )

    lenstool_file =  field[:-5]+".lenstool"
    rtl.rrg_to_lenstool( sheared_cat, field)
  




import numpy as np
import acs_limits as al
import RRGtools as at
import pyfits as py
import ipdb as pdb
def drizzle_position(      drizzle_file,
                           individual_files,
                           moments,
                           dataDir='.'):
    
    '''
    ;PURPOSE : RETURN THE POSITIONS OF EACH POINT IN THE DRIZZLED
    ;          IMAGE IN THE FRAME OF REFERENCE OF THE INDIVIDUAL EXP
    
    ;INPUTS : CLUSTER & FILTER : THE NAME AND FILTER OF CLUSTER
    ;         ASTROMETRYARR : ARRAY OF THE ASTROMETRY STRUCTURE FOR EACH
    ;         IMAGE 
    
    ;KEYWORD : NGRID : SQRT OF THE NUMBER O FPOINTS IN THE FINAL PSF GRID 
    ;          PSFGRID[X,Y] = THE GRID ON WHICH THE DRIZZLED IMAGE LAID
    ;          DRIZZLEDIMAGEORIENTATION : THE ORIENTATION OF THE DRIZZLED
    ;                                     IMAGE WRT WCS
    ;          [X,Y]_IMAGE : VECTORS OF THE INPUT X AND Y IF NOT WANT A 
    ;                        GRID IN THE FRAME OF THE DRIZZLED IMAGE
    ;          
    
    ;OPTIONAL OUTPUTS : NUM_EXPOSURES : NUMBER OF EXPOSURES EACH GALAXY HAS
    
    ;OUTPUTS : InDrizzleFrame : ARRAY OF NIMAGES X NX X NY X 3 
    ;                           where [*, 0, *] = xpositions
    ;                                 [*, 0, *] = ypositions
    ;                                 [*, 2, *] = is it in the image?
    
    ;UPDATES : 7/8/2013 : ALLOW RETURN OF DRIZZLED IMAGE ORIENTATION TO
    ;                     WCS
    ;          24/9/2013 : Returning keyword, num_exposures
    '''
    
    nGalaxies = len(moments.X_IMAGE)
    nImages = len( individual_files )

    InDrizzleFrame= np.zeros( (nImages, 3, nGalaxies**2))
    
    drizzle_obj = py.open(dataDir+'/'+drizzle_file)
    ImageData = drizzle_obj[0].data
    header =  drizzle_obj[0].header
  
    #Get orientation of the drizzled image
    drizzle_orientation=header['ORIENTAT']  
  
    Dimensions = ImageData.shape
    
    Orientation = np.zeros( nImages )
 
    InDrizzleFrame= np.zeros(( nImages,  3, nGalaxies) )

    #Now convert into wcs
    
    ra, dec = at.pix2deg( drizzle_file, moments.X_IMAGE, moments.Y_IMAGE)

    newcol = []
    for iImage in xrange(nImages):
        print( "%i/%i" %(iImage+1, nImages))
        #now see where each of our positions lie on each of the individual images
        
        SingleImageX, SingleImageY = at.deg2pix( individual_files[iImage], ra, dec)    
        #Get the limits of the field and find which one are
        #in the FOV and on the chip
        isin, image_orientation = al.acs_limits( SingleImageX, SingleImageY, \
                                                 individual_files[iImage])

        
        Orientation = np.zeros( len(SingleImageX)) +  image_orientation - drizzle_orientation
     
        isinArr= np.zeros( len( moments.X_IMAGE) )
        
        isinArr[ isin ] = 1
     
     

        InDrizzleFrame[iImage, 0, :] = SingleImageX
        InDrizzleFrame[iImage, 1, :] = SingleImageY
        InDrizzleFrame[iImage, 2, :] = isinArr

        
        x_column = py.Column( name=individual_files[iImage]+'_X_IMAGE', \
                                format=SingleImageX.dtype, \
                                array=SingleImageX )

        y_column = py.Column( name=individual_files[iImage]+'_Y_IMAGE', \
                                format=SingleImageY.dtype, \
                                array=SingleImageY )
        
        inFrame = py.Column( name=individual_files[iImage]+'_INFRAME', \
                                format=isinArr.dtype, \
                                array=isinArr )
                                
        orientat = py.Column( name=individual_files[iImage]+'_ORIENTAT', \
                                format=Orientation.dtype, \
                                array=Orientation )

        newcol.append( x_column )
        newcol.append( y_column )
        newcol.append( inFrame )
        newcol.append( orientat )
        
    orig_cols = moments.columns
    new_cols = py.ColDefs(newcol)
    
    hdu = py.BinTableHDU.from_columns(orig_cols + new_cols)
    
    num_exposures = np.sum(InDrizzleFrame[:, 2, :], axis=0).shape

    return hdu.data



   

   



import numpy as np
import os as os
def return_dirs( ):
    dirs = directories('.','.','.','.','.')
    dirs.get_dirs()
    return dirs

class directories( dict ):
    def __init__( self, data_dir, sex_files, psf_model_dir, code_dir, stilts_dir):
        self.__dict__['sex_files'] = sex_files
        self.__dict__['data_dir'] = data_dir
        self.__dict__['psf_model_dir'] = psf_model_dir
        self.__dict__['code_dir'] = code_dir
        self.__dict__['stilts_dir'] = stilts_dir

    def write_dirs( self ):
        file_obj = open('directories.cat',"wb")
        file_obj.write("DATA_DIR: %s \n" %self.data_dir)
        file_obj.write("SEX_FILES: %s \n" %self.sex_files)
        file_obj.write("PSF_MODEL_DIR: %s \n" %self.psf_model_dir)
        file_obj.write("CODE_DIR: %s \n" %self.code_dir)
        file_obj.write("STILTS_DIR: %s \n" %self.stilts_dir)

    def get_dirs( self ):
        dtypes = [('NAME', object), ('PATH', object)]
        directories = np.loadtxt('directories.cat',
                                     dtype=dtypes)
        
        self.__dict__['data_dir'] = directories['PATH'][0]
        self.__dict__['sex_files'] = directories['PATH'][1]
        self.__dict__['psf_model_dir'] = directories['PATH'][2]
        self.__dict__['code_dir'] = directories['PATH'][3]
        self.__dict__['stilts_dir'] = directories['PATH'][4]

    def check_dirs( self ):
        keys = self.keys()
        for iKey in keys:
            if not (os.path.isdir(self.__dict__[iKey])):
                raise ValueError('Cant find directory, ensure the path is correct (%s)' % self.__dict__[iKey])
            

            

    def keys(self):
        return self.__dict__.keys()

    def __getitem__(self, key): 
        return self.__dict__[key]




from matplotlib import pyplot as plt
import pyfits as py
import numpy as np
from matplotlib import gridspec as gridspec
import ipdb as pdb
def plot_shears( moments_catalogue, nbins=None,
                 min_gals_per_bins=100., catalogue=None ):
    '''

    Plot a quiver plot of the 'gamma1' and 'gamma2'


    '''


    fig = plt.figure()
    gs = gridspec.GridSpec( 1, 1)
    ax1 = plt.subplot( gs[0,0])

    if catalogue is None:
        catalogue = py.open( moments_catalogue )[1].data
    nGalaxies = len( catalogue.x)

    
 
    
    if nbins is None:
        nbins = np.int(nGalaxies / min_gals_per_bins)
    xbins = np.linspace( np.min(catalogue.x), np.max(catalogue.x), nbins + 1)
    ybins = np.linspace( np.min(catalogue.y), np.max(catalogue.y), nbins + 1)
    
    e1_map = np.zeros( (nbins, nbins) )
    e2_map = np.zeros( (nbins, nbins) )
    theta_map = np.zeros( (nbins, nbins) )


    xgrid, ygrid = np.meshgrid( xbins[:-1] - np.mean( xbins), ybins[:-1] - np.mean( ybins) )

        
    for i in xrange( nbins ):
        for j in xrange( nbins ):
            in_bin = (catalogue.x > xbins[i] ) & \
              (catalogue.x < xbins[i+1] ) &\
              (catalogue.y > ybins[j] ) & \
              (catalogue.y < ybins[j+1] )

            theta = np.arctan2( np.mean(catalogue.gamma2[ in_bin ]),
                                np.mean(catalogue.gamma1[ in_bin ]) )/2.
            
            
            gamma = np.sqrt( np.mean(catalogue.gamma1[ in_bin ])**2 +
                             np.mean(catalogue.gamma2[ in_bin ])**2 )
            
            e1_map[ j, i] = gamma*np.cos(theta) #np.mean(catalogue.gamma1[ in_bin ])
            e2_map[ j, i] = gamma*np.sin(theta) #np.mean(catalogue.gamma2[ in_bin ])
            theta_map[ j, i] = theta
    
    
    

    quiveropts = dict(color='white', headlength=0, pivot='middle', 
                    linewidth=.5, units='xy', width=.5, headwidth=1) 
    ax1.quiver( xgrid, ygrid, e1_map, e2_map, headaxislength=0, **quiveropts )
    plt.show()

    #bin the x and y
    



import numpy as np
import ipdb as pdb
import acs_map_xy as acs_map
def acs_3dpsf_basisfunctions( degree, x, y, focus ):




    # Generate relevant basis functions
    n_stars=np.max( np.array([len(x),len(y),len(focus)]))

    basis_function_order=np.zeros((1,3)) # All zeros
    for k in xrange(degree[2]+1):
        for j in xrange(degree[1]+1):
            for i in xrange(degree[0]+1):
                if (i+j+k > 0) & ((i+j) <= np.max(degree[0:2])):
                    
                    basis_function_order=np.vstack((basis_function_order, [i,j,k]))
    
    n_basis_functions= basis_function_order.shape[0]
    basis_function_value = np.zeros( (n_basis_functions, n_stars))
    
    for i in xrange(n_basis_functions):
        basis_function_value[i,:] = x**basis_function_order[i,0]*\
            y**basis_function_order[i,1] * \
            focus**basis_function_order[i,2]
                    
 

    return basis_function_value




# **********************************************************************
# **********************************************************************
# **********************************************************************

def acs_3dpsf_fit( scat, degree=np.array([3,2,2]), 
                   mag_cut=np.array([20.5,22]),
                   e_cut=1, size_cut=np.array([-np.inf,3])
                   ):

    # Fit the PSF from data in a SCAT catalogue
    # F814 I magnitude catalogue cut 
    

    if len(degree) < 3 :
        print "DEGREE must be 3D"
    degree[ degree > 0 ] = np.min(degree[ degree > 0 ])

    # Find the line dividing CCDs 1 and 2
    ccd_boundary = acs_map.acs_map_xy( np.array([0, 4095, 0, 4095]),
                                    np.array([2047, 2047, 2048, 2048]),
                                    pixel_scale=scat.pixscale)
    
    x1=np.mean([ccd_boundary.x[0],ccd_boundary.x[2]])
    x2=np.mean([ccd_boundary.x[1],ccd_boundary.x[3]])
    y1=np.mean([ccd_boundary.y[0],ccd_boundary.y[2]])
    y2=np.mean([ccd_boundary.y[1],ccd_boundary.y[3]])
    
    ccd_boundary_x1=np.mean([ccd_boundary.x[0],ccd_boundary.x[2]])
    ccd_boundary_x2=np.mean([ccd_boundary.x[1],ccd_boundary.x[3]])
    ccd_boundary_y1=np.mean([ccd_boundary.y[0],ccd_boundary.y[2]])
    ccd_boundary_y2=np.mean([ccd_boundary.y[1],ccd_boundary.y[3]])
    
    ccd_boundary_m=(ccd_boundary_y2-ccd_boundary_y1)/(ccd_boundary_x2-ccd_boundary_x1)
    ccd_boundary_c=ccd_boundary_y1-ccd_boundary_m*ccd_boundary_x1  


    # Find the centre of each CCD
    ccd_centre = acs_map.acs_map_xy( np.array([2048,2048]),
                                     np.array([3072,1024]), pixel_scale=scat.pixscale)


    # Select only the well-behaved stars
    good= np.isfinite(scat.field_focus[0][scat.field_id[0]]) & \
        np.isfinite(scat.e1_uncor_unrot[0]) & \
        np.isfinite(scat.e2_uncor_unrot[0]) & \
        np.isfinite(scat.xx_uncor[0]) & \
        np.isfinite(scat.xy_uncor[0]) & \
        np.isfinite(scat.yy_uncor[0]) & \
        np.isfinite(scat.xxxx_uncor[0]) & \
        np.isfinite(scat.xxxy_uncor[0]) & \
        np.isfinite(scat.xxyy_uncor[0]) & \
        np.isfinite(scat.xyyy_uncor[0]) & \
        np.isfinite(scat.yyyy_uncor[0])

    n_good = len(np.arange( len( good ))[good])
    print "Found a total of "+str(len(scat.x[0]))+" real stars, of which "+str(n_good)+" look well-behaved"
    

    # Store quantities to be fitted in local variables
    x=scat.x[0][good]
    y=scat.y[0][good]
    focus=scat.field_focus[0][scat.field_id[0]][good]
    ixx=scat.xx_uncor[0][good]
    ixy=scat.xy_uncor[0][good]
    iyy=scat.yy_uncor[0][good]
    ixxxx=scat.xxxx_uncor[0][good]
    ixxxy=scat.xxxy_uncor[0][good]
    ixxyy=scat.xxyy_uncor[0][good]
    ixyyy=scat.xyyy_uncor[0][good]
    iyyyy=scat.yyyy_uncor[0][good]
    e1=scat.e1_uncor_unrot[0][good]
    e2=scat.e2_uncor_unrot[0][good]
    
    # Work on each CCD separately
    init_coeffs_flag = True
    for ccd in xrange(2):
    
        # Report which CCD is being considered
        if ccd +1  == 1: 
            in_ccd = np.arange(len(y))[ y >= ccd_boundary_m*x+ccd_boundary_c]
            n_in_CCD = len(in_ccd)
        if ccd + 1 == 2:
            in_ccd = np.arange(len( y))[ y < ccd_boundary_m*x+ccd_boundary_c]
            n_in_CCD = len(in_ccd)
  
  
        if n_in_CCD > 0:
            #Compute matrix necessary for matrix inversion
            
            print "Fitting moments of "+str(n_in_CCD)+" real stars in CCD#"+str(ccd+1)
            basis_function_value=acs_3dpsf_basisfunctions(degree, 
                                                  x[in_ccd]-ccd_centre.x[ccd], 
                                                  y[in_ccd]-ccd_centre.y[ccd], 
                                                  focus[in_ccd])
           
            ls_matrix = np.dot( np.linalg.inv(np.dot(basis_function_value, basis_function_value.T)), basis_function_value)
        # Create global arrays to contain the answers
    
        n_basis_functions=np.shape(np.array(ls_matrix))[0]

        if init_coeffs_flag:
            acs_3dpsf_coeffs=basis_coeffs( ccd_centre,
                                    ccd_boundary_m, ccd_boundary_c,
                                    n_basis_functions, degree )
            init_coeffs_flag = False 
        print "Using "+str(n_basis_functions)+" basis functions"
        
        
        # Fit data to basis functions using least-squares inversion
        #these are all matrices
        acs_3dpsf_coeffs.ixx_fit[ccd, :]   = np.dot(ls_matrix ,ixx[in_ccd])
        acs_3dpsf_coeffs.ixy_fit[ccd, :]   = np.dot(ls_matrix , ixy[in_ccd])
        acs_3dpsf_coeffs.iyy_fit[ccd, :]   = np.dot(ls_matrix , iyy[in_ccd])
        acs_3dpsf_coeffs.ixxxx_fit[ccd, :] = np.dot(ls_matrix , ixxxx[in_ccd])
        acs_3dpsf_coeffs.ixxxy_fit[ccd, :] = np.dot(ls_matrix , ixxxy[in_ccd])
        acs_3dpsf_coeffs.ixxyy_fit[ccd, :] = np.dot(ls_matrix , ixxyy[in_ccd])
        acs_3dpsf_coeffs.ixyyy_fit[ccd, :] = np.dot(ls_matrix , ixyyy[in_ccd])
        acs_3dpsf_coeffs.iyyyy_fit[ccd, :] = np.dot(ls_matrix , iyyyy[in_ccd])
        acs_3dpsf_coeffs.e1_fit[ccd, :]    = np.dot(ls_matrix , e1[in_ccd])
        acs_3dpsf_coeffs.e2_fit[ccd, :]    = np.dot(ls_matrix , e2[in_ccd])
        
    else:
        print "No real stars found in CCD#"+str(ccd +1)
        


    return acs_3dpsf_coeffs



# **********************************************************************
# **********************************************************************
# **********************************************************************

def acs_3dpsf_reconstruct( acs_3dpsf_coeffs, x, y, focus, radius=None):


    # Create arrays to contain the final answer
    n_galaxies=np.max( np.array([len(x), len(y), len(focus)]) )
    
    if len(focus) == 1:
        focus_local = np.zeros(len(n_galaxies)) + focus
    else:
        focus_local=focus
        
    print "Found a total of "+str(n_galaxies)+" galaxies"
    if radius is None:
        radius=np.zeros(len(n_galaxies))+6
        
    moms=moments( x, y, radius[:n_galaxies],
                     acs_3dpsf_coeffs.degree )

        
    for ccd in xrange(2):
        #Report which CCD is being considered

        if ccd +1  == 1: 
            in_ccd = np.arange(len( y))[ y >= acs_3dpsf_coeffs.ccd_boundary_m*x+acs_3dpsf_coeffs.ccd_boundary_c]
            n_in_CCD = len(in_ccd)
        if ccd + 1 == 2:
            in_ccd = np.arange(len( y))[ y < acs_3dpsf_coeffs.ccd_boundary_m*x+acs_3dpsf_coeffs.ccd_boundary_c]
            n_in_CCD = len(in_ccd)
        
        if n_in_CCD > 0:
            print "Interpolating model PSF moments to the position of "+str(n_in_CCD)+" galaxies in CCD#"+str(ccd+1)

            #Fit the PSF
            basis_function_value=acs_3dpsf_basisfunctions(acs_3dpsf_coeffs.degree[0], \
                                                           x[in_ccd]-acs_3dpsf_coeffs.ccd_centre.x[ccd], \
                                                           y[in_ccd]-acs_3dpsf_coeffs.ccd_centre.y[ccd], \
                                                           focus_local[in_ccd] )
            
            moms.xx[in_ccd]   = np.dot(acs_3dpsf_coeffs.ixx_fit[ccd, :], basis_function_value)
            moms.xy[in_ccd]   =  np.dot(acs_3dpsf_coeffs.ixy_fit[ccd, :], basis_function_value)
            moms.yy[in_ccd]   =  np.dot(acs_3dpsf_coeffs.iyy_fit[ccd, :], basis_function_value)
            moms.xxxx[in_ccd] =  np.dot(acs_3dpsf_coeffs.ixxxx_fit[ccd, :], basis_function_value)
            moms.xxxy[in_ccd] =  np.dot(acs_3dpsf_coeffs.ixxxy_fit[ccd, :], basis_function_value)
            moms.xxyy[in_ccd] =  np.dot(acs_3dpsf_coeffs.ixxyy_fit[ccd, :], basis_function_value)
            moms.xyyy[in_ccd] =  np.dot(acs_3dpsf_coeffs.ixyyy_fit[ccd, :], basis_function_value)
            moms.yyyy[in_ccd] =  np.dot(acs_3dpsf_coeffs.iyyyy_fit[ccd, :], basis_function_value)
            moms.e1[in_ccd]   =  np.dot(acs_3dpsf_coeffs.e1_fit[ccd, :], basis_function_value)
            moms.e2[in_ccd]   =  np.dot(acs_3dpsf_coeffs.e2_fit[ccd, :], basis_function_value)
            
           
        else:
            print "No galaxies in CCD#"+str(ccd)

    # Work out PSF ellipticities at positions of galaxies properly. Tsk!
    moms.e1 = (moms.xx-moms.yy)/(moms.xx+moms.yy)
    moms.e2 = 2*moms.xy/(moms.xx+moms.yy)

    return moms


# **********************************************************************
# **********************************************************************
# **********************************************************************

def acs_3dpsf( x, y, focus, radius, scat, 
                acs_3dpsf_coeffs=None, 
                degree=np.array([3,2,2])):

    # Fit the PSF
    if acs_3dpsf_coeffs is None:
        acs_3dpsf_coeffs=acs_3dpsf_fit(scat, degree=degree)
    #Reconstruct the PSF
    acs_moms=acs_3dpsf_reconstruct(acs_3dpsf_coeffs, x, y, focus, radius)
    return acs_moms
  


class basis_coeffs:

    def __init__( self, ccd_centre, ccd_boundary_m, \
                  ccd_boundary_c, n_basis_functions, degree ):
        self.degree = degree,
        self.ccd_centre = ccd_centre
        self.ccd_boundary_m = ccd_boundary_m
        self.ccd_boundary_c = ccd_boundary_c
        self.ixx_fit = np.zeros((2,n_basis_functions))
        self.ixy_fit = np.zeros((2,n_basis_functions))
        self.iyy_fit = np.zeros((2,n_basis_functions))
        self.ixxxx_fit = np.zeros((2,n_basis_functions))
        self.ixxxy_fit = np.zeros((2,n_basis_functions))
        self.ixxyy_fit = np.zeros((2,n_basis_functions))
        self.ixyyy_fit = np.zeros((2,n_basis_functions))
        self.iyyyy_fit = np.zeros((2,n_basis_functions))
        self.e1_fit = np.zeros((2,n_basis_functions))
        self.e2_fit = np.zeros((2,n_basis_functions))
    
    
class moments( dict ):

    def __init__(self, x, y, radius, degree ):
        n_objects = len(x)
        self.__dict__['x'] = x
        self.__dict__['y'] = y
        self.__dict__['e1']=np.zeros(n_objects)
        self.__dict__['e2']=np.zeros(n_objects)
        self.__dict__['xx']=np.zeros(n_objects)
        self.__dict__['xy']=np.zeros(n_objects)
        self.__dict__['yy']=np.zeros(n_objects)
        self.__dict__['xxxx']=np.zeros(n_objects)
        self.__dict__['xxxy']=np.zeros(n_objects)
        self.__dict__['xxyy']=np.zeros(n_objects)
        self.__dict__['xyyy']=np.zeros(n_objects)
        self.__dict__['yyyy']=np.zeros(n_objects)
        self.__dict__['radius'] = radius
        self.__dict__['degree'] = degree


    def keys(self):
        return self.__dict__.keys()

    def __getitem__(self, key): 
        return self.__dict__[key]




import mmm as mmm
import pyfits as py
import numpy as np
import ipdb as pdb
from matplotlib import pyplot as plt
import RRGtools as at
def measure_moms(fits_image, sex_catalog, outfile,
                     width=None,
                     saturation=800000000, badval=-99,
                     cut_off=2.5, fieldback=0, mult=1,
                     min_rad=1.5, startx=0, starty=0,
                     silent=0, weight_image=None,
                     wt_ext=0, nocenter=0,
                     sgm_im='null',
                     object_catalogue=None,
                     bad_val=-99, regfile=None,
                     skymed=None, skysd=None,
                     return_moms=True, **kwargs):
                     
    '''
    ;
    ; NAME:                  rrg_measure_moms
    ;
    ; PURPOSE:
    ;    Uses the RRG method to measure the moments of objects in a fits image.  
    
    ;
    ; INPUTS:
    ;   fits_image- fits image with single extension
    ;   sex_catalog-SExtractor catalog in fits format
    ;   outfile- output file to store IDL array of variables
    ;
    ; OUTPUTS:
    ; stores moments and other info in the output file as an idl structure  
    ;
    ; KEYWORD PARAMETERS:
    ;   width- gaussian window function width, if not input is calculated from area
    ;   saturation,bad_value are saturation level and value of pixels not to use (outside image)
    ;   cut_off- cut_off for measuring objects default is 2.5x width
    ;   fieldback -1 means use a field backgroup as calculated by IDL SKY, 0(default) uses object back from SEx
    ;   mult- multiplier used to find gaussian width from area
    ;   min_rad- stellar radius
    ;   startx, starty- lower left corner of image in pixel values, default to zero
    ;   silent set to 1 to supress messages
    ;   stellar set to 1 to evaluate all objects with the same radius (min_rad)
    ;   weight=use a weight image to determine good/bad pixels (yes=1 0=no/default)
    ;   wt_ext= extension of weight image in fits file
    ;  nocenter= if 1 then uses Sextractor centers and doesnt centroid
    ;  name_weight_im=name of the weight image
    ;  sgm_im Sextractor segmentation image to get sky background 
    ;  souces = predetermined sources to find
    ;MODIFICATION HISTORY:
    ;    30 Dec 2002 jrhodes
    ;    11 March 2003 jrhodes
    

    IF N_PARAMS(0) LT 1 THEN BEGIN
    PRINT, 'rrg_measure_moms,fits_image,sex_catalog,outfile,width=width,saturation=saturation,'
    PRINT, '    badval=badval,cut_off=cut_off,fieldback=fieldback,mult=mult,min_rad=min_rad,'
    PRINT, '    startx=startx,starty=starty,silent=silent,stellar=stellar,'
    PRINT, '    weight=weight,wt_ext=wt_ext,nocenter=nocenter,name_weight_im=name_weight_im'
    return
    ENDIF
    ;
    '''
                     
    img_file = py.open( fits_image )
    img = img_file[0].data
    imhead = img_file[0].header
    exp_time = imhead['EXPTIME']

    
    #What is mmm?
    if (skymed is None )| \
        (skysd is None):
        skymed, skysd, skysw = mmm.mmm(img)
    
    print(' % f skymed and %f skysd' % (skymed,skysd))
    
    if object_catalogue is None:
        cat_file=py.open(sex_catalog)
        object_catalogue = cat_file[1].data
        header=cat_file[1].header
     
    nGalaxies = len(object_catalogue['RA'])

    
    try:
        if not 'xGal' in kwargs:
            xGal = object_catalogue['X_IMAGE']
        else:
            xGal = kwargs['xGal']
        if not 'yGal' in kwargs:
            yGal = object_catalogue['Y_IMAGE']
        else:
            yGal = kwargs['yGal']
    except:
        xGal, yGal = at.deg2pix( fits_image, object_catalogue['RA'], object_catalogue['DEC'] )

  
    ysize= imhead['NAXIS2']
    xsize= imhead['NAXIS1']

    if weight_image is None:
        wt_image = np.ones( img.shape)
    else:
        wt_image = py.open( weight_image )[0].data



    #The background for each galaxy. if none take from sex cata;pgue
    try:
        back=object_catalogue.BACKGROUND
    except:
        back =  np.zeros( nGalaxies) + skysd

    

    if (sgm_im != 'null') & (fieldback == 1):
        seg_file = py.open(sgm_im)
        seg = seg_file[0].data
        shdr = seg_file[0].header
        '''
        sel=where((seg eq 0)and (wt_image gt 0))
        mmm,img(sel),seg_sky,seg_skysd
        back=replicate(seg_sky,num) 
        '''
    if not 'radius' in kwargs:
        area=np.pi*object_catalogue['A_IMAGE']*object_catalogue['B_IMAGE']
        radius = np.sqrt( area / np.pi )*mult
    else:
        radius = kwargs['radius']
    
    radius[ radius < min_rad ] = min_rad

    
    if width is not None:
        radius=np.zeros(nGalaxies) + width
        
    cut_rad = radius*cut_off # cut off radius of object
    offedge=0 #;number of objects off the endge initially
    centerprob=0 # number of objects with centroiding problems
    badpix_prob=0 #number of objects with bad pixels

    
    galaxy_moments = moms( nGalaxies, radius=radius )
    
    for i in xrange( nGalaxies ):
        
        #following changed by jrhodes to account for different indexing in SExtractor and IDL
       

        deltax=1.0
        deltay=1.0
        count=1
        sum_int=0
        int_moms = moms( 1 )
        xc=xGal[i]-0.5
        yc=yGal[i]-0.5
        
        blank=0
        go_on=1
        badpix_centroid='no'
    
        if (xc-cut_rad[i]-1.< startx) | \
            (yc-cut_rad[i]-1 < starty) | \
            (xc+cut_rad[i]+1 > xsize ) | \
            (yc+cut_rad[i]+1 > ysize ):

            if not silent:
                print(' %i %f %f too close to edge at iteration 1\n' % (i,xGal[i],yGal[i]))
            offedge=offedge+1
            go_on=0
            galaxy_moments.prob[i] += 1
            

        if nocenter == 0:
            while  ((np.abs(deltax) > 0.01) |  \
              (np.abs(deltay) > 0.01)) & \
               (count < 500) & (go_on == 1) :
                #These are needed for all moments
                #Cut out a postage stamp of the image
                begin_x = np.round(xc-cut_rad[i]-1).astype(int)
                begin_y = np.round(yc-cut_rad[i]-1).astype(int)
                end_x = np.round(xc+cut_rad[i]+2).astype(int)
                end_y = np.round(yc+cut_rad[i]+2).astype(int)
            
            
                xvec = np.arange(begin_x, end_x)
                yvec = np.arange(begin_y, end_y)
            
                #this will be used throughotu the loop as well
                uncut_xgrid, uncut_ygrid = np.meshgrid( xvec, yvec)
                
                #Dist will also be used throughout the loop
                uncut_dist = np.sqrt( (uncut_xgrid - xc + 0.5)**2 + \
                                    (uncut_ygrid-yc+0.5)**2)
            
                #Also cut the images at the cut_radi
                dist = uncut_dist[uncut_dist < cut_rad[i]]
                xgrid = uncut_xgrid[uncut_dist < cut_rad[i]]
                ygrid = uncut_ygrid[uncut_dist < cut_rad[i]]
    
                #These will be used throughout the loop
                uncut_postage_stamp = img[begin_y:end_y, begin_x:end_x]
                postage_stamp = uncut_postage_stamp[uncut_dist < cut_rad[i]]
            
                uncut_postage_weight = wt_image[begin_y:end_y, begin_x:end_x]
                postage_weight = uncut_postage_weight[uncut_dist < cut_rad[i]]
    
                #This might be
                g_f = np.exp(-(dist**2)/(2.*radius[i]**2))
                weight_gal = g_f*( postage_stamp -  back[i])
                sum_int = np.sum( weight_gal )
                

                #Then finally change the dist ]
                #find the centroid
 
   
                #changed from above line for SNAP sims jrhodes 7/09/07
                #used below for SNAP sims
                checkx=xc
                checky=yc
                
                #Get the centre
                xc = np.sum( weight_gal*(xgrid+0.5))/sum_int
                yc = np.sum( weight_gal*(ygrid+0.5))/sum_int
                
                count += 1
                deltax=xc-checkx
                deltay=yc-checky;
            
                if ( xc-cut_rad[i]-1 < startx) | \
                    ( yc-cut_rad[i]-1 < starty) | \
                    ( xc+cut_rad[i]+1 > xsize ) | \
                    ( yc+cut_rad[i]+1 > ysize) :
                    if not silent:
                        print( ' %i %f %f too close to edge at iteration %i' % \
                            (i,xGal[i],yGal[i],(count-1)) )
                   
                    go_on=0
                    galaxy_moments.prob[i] += 1
    
                if (badpix_centroid == 'yes'):
                    go_on=0
                    if not silent:
                        print( '%i %f %f  bad pix in centroid at iteration %f' %
                                    (i,xGal[i],yGal[i],(count-1)) )
                                
                    galaxy_moments.prob[i] += 4
                    badpix_prob += 1
                        #End this if
                    #Ends the xc_ci of
                #this ends the while
            if (count > 100):
                centerprob=centerprob+1
                if not silent:
                    print( '%i %f %f Too many centering iterations %i' % \
                            (i,xGal[i],yGal[i],count))
                galaxy_moments.prob[i] += 2
                
            if np.sqrt( (xc-xGal[i])**2 + (yc-yGal[i])**2) > mult*radius[i]:
                go_on=0
                centerprob += 1
                if not silent:
                    print( '%i %f %f Centroid shift too large ' % (i,xGal[i],yGal[i]) )
                
                galaxy_moments.prob[i] += 2
      
        
            #this ends the centre befor the while
            #Now back into normal main galaxy loop
        badpix_mom='no'    
    # check for saturated pixels and bad pixels
        int_moms.x = xc
        int_moms.y = yc
       
        if go_on == 1:


            if (np.any(postage_stamp > saturation)) | \
                (np.any(postage_stamp == bad_val)) | \
                (np.any(postage_weight == 0)):
                go_on = 0
                galaxy_moments.prob[i] += 4
                badpix_mom='yes'
                
            if badpix_mom == 'yes':
                badpix_prob += 1 
                
                print('%i %f %f Bad pixel(s) in centroiding' %\
                        (i,xGal[i],yGal[i]))

        
    #find moments and center error
    

        if go_on == 1:
            rel_xgrid = xgrid + 0.5 - int_moms.x
            rel_ygrid = ygrid + 0.5 - int_moms.y
            
            int_moms.xx = np.sum(rel_xgrid*rel_xgrid*weight_gal) / sum_int
            int_moms.yy = np.sum(rel_ygrid*rel_ygrid*weight_gal) / sum_int
            int_moms.xy = np.sum(rel_xgrid*rel_ygrid*weight_gal) / sum_int



            int_moms.xxxx = np.sum(weight_gal*(rel_xgrid**4)) / sum_int
            int_moms.xxxy = np.sum(weight_gal*(rel_xgrid**3)*rel_ygrid)  / sum_int
            int_moms.xxyy = np.sum(weight_gal*(rel_xgrid**2)*(rel_ygrid**2)) / sum_int
            int_moms.xyyy=  np.sum(weight_gal*(rel_ygrid**3)*rel_xgrid)/ sum_int
            int_moms.yyyy = np.sum(weight_gal*(rel_ygrid**4)) / sum_int

         
           

            
            
            #find moment errors and covariances

            #Error in internsity
            I_err_sq =  skysd**2+(postage_stamp-back[i])/( exp_time) / sum_int**2

            #Error in x and y
            sum_xxdI = np.sum(g_f*g_f*I_err_sq*rel_xgrid*rel_xgrid) / sum_int
            sum_yydI = np.sum(g_f*g_f*I_err_sq*rel_ygrid*rel_ygrid) / sum_int
            int_moms.error.x = np.sqrt(sum_xxdI)/sum_int
            int_moms.error.y = np.sqrt(sum_yydI)/sum_int

            #Error in second order
            int_moms.error.xx = np.sum(g_f*g_f*I_err_sq*(rel_xgrid**2-int_moms.xx)**2)/sum_int
            int_moms.error.yy = np.sum(g_f*g_f*I_err_sq*(rel_ygrid**2-int_moms.yy)**2)/sum_int
            int_moms.error.xy = np.sum(g_f*g_f*I_err_sq*(rel_xgrid*rel_ygrid-int_moms.xy)**2)/sum_int

            #error in 4th order
            int_moms.error.xxyy = np.sum(g_f*g_f*I_err_sq*\
                (rel_xgrid*rel_xgrid-int_moms.xx)*\
                (rel_ygrid*rel_ygrid-int_moms.yy))/sum_int**2
                          
            int_moms.error.xxxy = np.sum(g_f*g_f*I_err_sq*\
                (rel_xgrid*rel_xgrid-int_moms.xx)*\
                (rel_ygrid*rel_xgrid-int_moms.yy))/sum_int**2
                        
            int_moms.error.xyyy = np.sum(g_f*g_f*I_err_sq*\
                          (rel_xgrid*rel_xgrid-int_moms.xy)*\
                          (rel_ygrid*rel_ygrid-int_moms.yy))/sum_int**2
            
            
        else:
            
            int_moms.xx, int_moms.yy, int_moms.xy = [-99, -99, -99]
            int_moms.xxxx, int_moms.xxxy =  [0, 0]
            int_moms.xxyy, int_moms.xyyy, int_moms.yyyy = [0, 0, 0]
            int_moms.error.xx, int_moms.error.yy, int_moms.error.xy = [0, 0, 0]
            int_moms.error.xxxx, int_moms.error.xxxy =  [0, 0]
            int_moms.error.xxyy, int_moms.error.xyyy, int_moms.error.yyyy = [0, 0, 0]
            xc_err, yc_err = [0, 0]




        galaxy_moments.append( i, int_moms )
        
    #Append some needed things
    #Can just append the RA as this is not the same as the pixel position
    #Need to find the re-centred RA
    recentred_ra, recentred_dec = \
      at.pix2deg( fits_image, galaxy_moments.x, galaxy_moments.y)

    galaxy_moments.ra = recentred_ra
    galaxy_moments.dec = recentred_dec
   
    galaxy_moments.calc_e1e2( mult_rad=mult)     

    galaxy_moments.write_to_fits( object_catalogue, outfile )

    if regfile is not None:
        galaxy_moments.fits_to_ellipse( regfile)
    
    if return_moms:
        return py.open(outfile)[1].data

class moms( dict ):

    def __init__( self, ngalaxies, radius=None, mk_error=True ):
        self.__dict__['x'] = np.zeros( ngalaxies )
        self.__dict__['y'] = np.zeros( ngalaxies )
        self.__dict__['xx'] = np.zeros( ngalaxies )
        self.__dict__['yy'] = np.zeros( ngalaxies )
        self.__dict__['xy'] = np.zeros( ngalaxies )
        self.__dict__['xxxx'] = np.zeros( ngalaxies)
        self.__dict__['yyyy'] = np.zeros( ngalaxies)
        self.__dict__['xxyy'] = np.zeros( ngalaxies)
        self.__dict__['xyyy'] = np.zeros( ngalaxies)
        self.__dict__['xxxy'] = np.zeros( ngalaxies)
        self.__dict__['prob'] = np.zeros( ngalaxies)
        if radius is not None:
            self.__dict__['radius'] = radius

        if mk_error:
            self.error = moms( ngalaxies, mk_error=False )
    
    
    def append( self, index, imom ):

        mom_names = self.keys()
        for i in mom_names:
            if i == 'radius':
                continue
            self[i][index] = imom[i]
            if i == 'error':
                error_names = imom.error.keys()
                for i in error_names:
                    self.error[i][index] = imom.error[i]


    def calc_e1e2( self, mult_rad=1 ):
        self.__dict__['e1']=(self.xx-self.yy)/ (self.xx+self.yy)
        self.e2=2.*self.xy/(self.xx+self.yy)

        self.__dict__['e1_err'] = np.sqrt( ((self.xx+self.yy)**(-2))*\
                               (self.error.xx**2*(1-self.e1)**2 + \
                                self.error.yy**2*(1+self.e1)**2-\
                                2.*(1-self.e1*self.e1)*self.error.xxyy))
                                
        self.__dict__['e2_err'] = np.sqrt( ((self.xx+self.yy)**(-2))*\
                               ((self.error.xx**2+self.error.yy**2+\
                                 2.*self.error.xxyy)*self.e2**2+\
                                 4.*(self.error.xy**2-self.e2*\
                                     (self.error.xxxy*self.error.xyyy))))

        self.__dict__['ell'] = np.sqrt( self.e1**2 + self.e2**2)
        self.__dict__['pa'] = 0.5*np.arctan2(self.e2, self.e1)*180./np.pi
        
        self.__dict__['gal_size'] = np.sqrt( (self.xx + self.yy)/2.)
        self.prob[ self.xx + self.yy < 0 ] += 5
        self.prob[ self.ell > 1.0 ] += 6
        self.__dict__['a'] = self.gal_size*(1.+self.ell)
        self.__dict__['b'] = self.gal_size*(1.-self.ell)
        

    def write_to_fits( self, sex_catalog, filename):
        '''
        Append the moments to input sex catalogue
        '''
        
        mom_names = self.keys()
        fits_columns = []
        
        for iColumn in mom_names:
            if iColumn == 'error':
                error_names = self.error.keys()
                for iColumn_err in error_names:
                    fits_columns.append(py.Column( name=iColumn_err+'_err',
                                                   format=self.error[iColumn_err].dtype,
                                                   array=self.error[iColumn_err] ))
            else:
                fits_columns.append(py.Column( name=iColumn, \
                                               format=self[iColumn].dtype, \
                                               array=self[iColumn] ))
        
        for iCatalog in sex_catalog.columns.names:
            if (iCatalog in mom_names) | ('err' in iCatalog):
                continue
            else:
                fits_columns.append(py.Column( name=iCatalog,
                                                   format=sex_catalog[iCatalog].dtype,
                                                   array=sex_catalog[iCatalog] ))

            
        fits_table = py.BinTableHDU.from_columns( fits_columns )
        fits_table.writeto( filename, clobber=True )



    def fits_to_ellipse( self, filename):

        regionFile = open( filename, "wb")
        regionFile.write('# Region file format: DS9 version 4.1\n')
        regionFile.write('# Filename: dummy.fits\n')
        regionFile.write("global color=green dashlist=8 3 width=1 font='helvetica 10 normal roman' select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1\n")
        regionFile.write("physical\n")
        
        for i in xrange(len(self.ra)):
            
            regionFile.write('ellipse(%0.4f,%0.4f,%0.4f,%0.4f,%0.4f)\n' %
                             (self.x[i], self.y[i], self.a[i], self.b[i], self.pa[i]))


        

    def keys(self):
        return self.__dict__.keys()
    
    def __getitem__(self, key): 
        return self.__dict__[key]



import numpy as np
import pyfits as py

import ipdb as pdb

def ellipse_to_reg( catalogue_name ):
    '''
    Creat a region file to check thta i have measured ellipses correctedly
    '''


    catalogue = py.open( catalogue_name )[1].data


    regionFile = open( 'ellipse.reg', 'wb')

    ellipticity = np.sqrt( catalogue.e1**2 +
                           catalogue.e2**2 )
    
    semi_major = (1 + ellipticity)*catalogue.gal_size
    semi_minor = (1 - ellipticity)*catalogue.gal_size
    angle = np.arctan2( catalogue.e2, catalogue.e1 )*180./np.pi/2.


    regionFile.write('# Region file format: DS9 version 4.1\n')
    regionFile.write('# Filename: dummy.fits\n')
    regionFile.write("global color=green dashlist=8 3 width=1 font='helvetica 10 normal roman' select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1\n")
    regionFile.write("physical\n")

    for iGal in xrange(len(catalogue.x)):
        
        regionFile.write('Ellipse(%0.4f,%0.4f,%0.4f,%0.4f,%0.4f)\n'
                    % (catalogue.x[iGal], catalogue.y[iGal],
                        semi_major[iGal], semi_minor[iGal], angle[iGal]))
    



#!/usr/bin/env python
# D. Jones - 2/13/14
"""This code is from the IDL Astronomy Users Library"""

import numpy as np

def mmm( sky_vector, 
         highbad = False,
         debug = False,
         readnoise = False,
         nsky = False,
         integer = "discrete",
         mxiter = 50,
         minsky = 20,
         nan=True):
    """Estimate the sky background in a stellar contaminated field.
    MMM assumes that contaminated sky pixel values overwhelmingly display 
    POSITIVE departures from the true value.  Adapted from DAOPHOT 
    routine of the same name.
    
    CALLING SEQUENCE:
         skymod,sigma,skew = mmm.mmm( sky, highbad= , readnoise=, debug=, 
                                      minsky=, nsky=, integer=)
    
    INPUTS:
         sky - Array or Vector containing sky values.  This version of
                MMM does not require SKY to be sorted beforehand.
    
    RETURNS:
         skymod - Scalar giving estimated mode of the sky values
         sigma -  Scalar giving standard deviation of the peak in the sky
                   histogram.  If for some reason it is impossible to derive
                   skymod, then SIGMA = -1.0
         skew -   Scalar giving skewness of the peak in the sky histogram
    
         If no output variables are supplied or if "debug" is set
         then the values of skymod, sigma and skew will be printed.
    
    OPTIONAL KEYWORD INPUTS:
         highbad - scalar value of the (lowest) "bad" pixel level (e.g. cosmic 
                    rays or saturated pixels) If not supplied, then there is 
                    assumed to be no high bad pixels.
         minsky - Integer giving mininum number of sky values to be used.   MMM
                    will return an error if fewer sky elements are supplied.
                    Default = 20.
         maxiter - integer giving maximum number of iterations allowed,default=50
         readnoise - Scalar giving the read noise (or minimum noise for any 
                     pixel).  Normally, MMM determines the (robust) median by 
                    averaging the central 20% of the sky values.  In some cases
                    where the noise is low, and pixel values are quantized a
                    larger fraction may be needed.  By supplying the optional
                    read noise parameter, MMM is better able to adjust the
                    fraction of pixels used to determine the median.                
         integer - Set this keyword if the  input SKY vector only contains
                    discrete integer values.  This keyword is only needed if the
                    SKY vector is of type float or double precision, but contains 
                    only discrete integer values.  (Prior to July 2004, the
                    equivalent of /INTEGER was set for all data types)
         debug -   If this keyword is set and non-zero, then additional 
                    information is displayed at the terminal.
    OPTIONAL OUTPUT KEYWORD:
         nsky - Integer scalar giving the number of pixels actually used for the
                 sky computation (after outliers have been removed).
    NOTES:
         (1) Program assumes that low "bad" pixels (e.g. bad CCD columns) have
              already been deleted from the SKY vector.
         (2) MMM was updated in June 2004 to better match more recent versions
              of DAOPHOT.
         (3) Does not work well in the limit of low Poisson integer counts
         (4) MMM may fail for strongly skewed distributions.
    METHOD:
         The algorithm used by MMM consists of roughly two parts:
           (1) The average and sigma of the sky pixels is computed.  These values
                are used to eliminate outliers, i.e. values with a low probability
                given a Gaussian with specified average and sigma.  The average
                and sigma are then recomputed and the process repeated up to 20
                iterations.
           (2) The amount of contamination by stars is estimated by comparing the 
                mean and median of the remaining sky pixels.  If the mean is larger
                than the median then the true sky value is estimated by
                3*median - 2*mean
             
     REVISION HISTORY:
           Adapted to IDL from 1986 version of DAOPHOT in STSDAS        W. Landsman, STX           Feb,      1987
           Added HIGHBAD keyword                                        W. Landsman                January,  1991
           Fixed occasional problem with integer inputs                 W. Landsman                Feb,      1994
           Avoid possible 16 bit integer overflow                       W. Landsman                November, 2001
           Added READNOISE, NSKY keywords,  new median computation      W. Landsman                June,     2004
           Added INTEGER keyword                                        W. Landsman                July,     2004
           Improve numerical precision                                  W. Landsman                October,  2004
           Fewer aborts on strange input sky histograms                 W. Landsman                October,  2005
           Added /SILENT keyword                                                                   November, 2005
           Fix too many /CON keywords to MESSAGE                        W.L.                       December, 2005
           Fix bug introduced June 2004 removing outliers               N. Cunningham/W. Landsman  January,  2006
            when READNOISE not set
           Make sure that MESSAGE never aborts                          W. Landsman                January,  2008
           Add mxiter keyword and change default to 50                  W. Landsman                August,   2011
           Added MINSKY keyword                                         W.L.                       December, 2011
           Converted to Python                                          D. Jones                   January,  2014
    """

    if nan: sky_vector = sky_vector[np.where(sky_vector == sky_vector)]
    nsky = len( sky_vector )            #Get number of sky elements 
 
    if nsky < minsky:
        sigma=-1.0 ;  skew = 0.0; skymod = np.nan
        print('ERROR -Input vector must contain at least '+str(minsky)+' elements')
        return(skymod,sigma,skew)
 
    nlast = nsky-1                        #Subscript of last pixel in SKY array
    if debug:
        print('Processing '+str(nsky) + ' element array')
    sz_sky = np.shape(sky_vector)

    sky = np.sort(sky_vector)    #Sort SKY in ascending values

    skymid = 0.5*sky[int((nsky-1)/2)] + 0.5*sky[int(nsky/2)]  #Median value of all sky values
       
    cut1 = np.min( [skymid-sky[0],sky[nsky-1] - skymid] ) 
    if highbad: 
        cut1[np.where(cut1 > highbad - skymid)[0]] = highbad - skymid
    cut2 = skymid + cut1
    cut1 = skymid - cut1

    # Select the pixels between Cut1 and Cut2

    good = np.where( (sky <= cut2) & (sky >= cut1))[0]
    Ngood = len(good)

    if ( Ngood == 0 ):
        sigma=-1.0 ;  skew = 0.0; skymod = 0.0   
        print('ERROR - No sky values fall within ' + str(cut1) + \
                     ' and ' + str(cut2))
        return(skymod,sigma,skew)
  
    delta = sky[good] - skymid  #Subtract median to improve arithmetic accuracy
    sum = np.sum(delta.astype('float64'))
    sumsq = np.sum(delta.astype('float64')**2)

    maximm = np.max( good) ; minimm = np.min(good)  # Highest value accepted at upper end of vector
    minimm = minimm -1               #Highest value reject at lower end of vector

    # Compute mean and sigma (from the first pass).

    skymed = 0.5*sky[(minimm+maximm+1)/2] + 0.5*sky[(minimm+maximm)/2 + 1] #median 
    skymn = sum/(maximm-minimm)                            #mean       
    sigma = np.sqrt(sumsq/(maximm-minimm)-skymn**2)             #sigma          
    skymn = skymn + skymid         #Add median which was subtracted off earlier 

    #    If mean is less than the mode, then the contamination is slight, and the
    #    mean value is what we really want.
#    skymod =  (skymed < skymn) ? 3.*skymed - 2.*skymn : skymn
    if skymed < skymn:
        skymod = 3.*skymed - 2.*skymn
    else: skymod = skymn

    # Rejection and recomputation loop:

    niter = 0
    clamp = 1
    old = 0
# START_LOOP:
    redo = True
    while redo:
        niter = niter + 1                     
        if ( niter > mxiter ):
            sigma=-1.0 ;  skew = 0.0   
            print('ERROR - Too many ('+str(mxiter) + ') iterations,' + \
                      ' unable to compute sky')
#            import pdb; pdb.set_trace()
            return(skymod,sigma,skew)

        if ( maximm-minimm < minsky ):    #Error? 

            sigma = -1.0 ;  skew = 0.0   
            print('ERROR - Too few ('+str(maximm-minimm) +  \
                      ') valid sky elements, unable to compute sky')
            return(skymod,sigma,skew)

        # Compute Chauvenet rejection criterion.

        r = np.log10( float( maximm-minimm ) )      
        r = np.max( [ 2., ( -0.1042*r + 1.1695)*r + 0.8895 ] )

        # Compute rejection limits (symmetric about the current mode).

        cut = r*sigma + 0.5*np.abs(skymn-skymod)   
    #    if integer: cut = cut > 1.5 
        cut1 = skymod - cut   ;    cut2 = skymod + cut

        # 
        # Recompute mean and sigma by adding and/or subtracting sky values
        # at both ends of the interval of acceptable values.
      
        redo = False
        newmin = minimm             
        if sky[newmin+1] >= cut1: tst_min = 1      #Is minimm+1 above current CUT?
        else: tst_min = 0
        if (newmin == -1) and tst_min: done = 1    #Are we at first pixel of SKY?
        else: done = 0
        if not done:
            if newmin > 0: skyind = newmin
            else: skyind = 0
            if (sky[skyind] < cut1) and tst_min: done = 1
        if not done:
            istep = 1 - 2*int(tst_min)
            while not done:
                newmin = newmin + istep
                if (newmin == -1) | (newmin == nlast): done = 1
                if not done:
                    if (sky[newmin] <= cut1) and (sky[newmin+1] >= cut1): done = 1
        
            if tst_min:  delta = sky[newmin+1:minimm+1] - skymid
            else: delta = sky[minimm+1:newmin+1] - skymid
            sum = sum - istep*np.sum(delta)
            sumsq = sumsq - istep*np.sum(delta**2)
            redo = True
            minimm = newmin

        newmax = maximm
        if sky[maximm] <= cut2: tst_max = 1           #Is current maximum below upper cut?
        else: tst_max = 0
        if (maximm == nlast) and tst_max: done = 1
        else: done = 0                  #Are we at last pixel of SKY array?
        if not done:
            if maximm+1 < nlast: skyind = maximm+1
            else: skyind = nlast
            if ( tst_max ) and (sky[skyind] > cut2): done = 1 
        if not done: # keep incrementing newmax
            istep = -1 + 2*int(tst_max)         #Increment up or down?
            while not done:
                newmax = newmax + istep
                if (newmax == nlast) or (newmax == -1): done = 1
                if not done:
                    if ( sky[newmax] <= cut2 ) and ( sky[newmax+1] >= cut2 ): done = 1

            if tst_max: 
                delta = sky[maximm+1:newmax+1] - skymid
            else:
                delta = sky[newmax+1:maximm+1] - skymid
            sum = sum + istep*np.sum(delta)
            sumsq = sumsq + istep*np.sum(delta**2)
            redo = True
            maximm = newmax

        #       
        # Compute mean and sigma (from this pass).
        #
        nsky = maximm - minimm
        if ( nsky < minsky ): # error?
            sigma = -1.0 ;  skew = 0.0   
            print('ERROR - Outlier rejection left too few sky elements')
            return(skymod,sigma,skew)

        skymn = sum/nsky
        var = sumsq/nsky - skymn**2
        if var < 0: var = 0
        sigma = float( np.sqrt( var ))
        skymn = skymn + skymid 

        #  Determine a more robust median by averaging the central 20% of pixels.
        #  Estimate the median using the mean of the central 20 percent of sky
        #  values.   Be careful to include a perfectly symmetric sample of pixels about
        #  the median, whether the total number is even or odd within the acceptance
        #  interval
    
        center = (minimm + 1 + maximm)/2.
        side = np.round(0.2*(maximm-minimm))/2.  + 0.25
        j = np.round(center-side)
        k = np.round(center+side)

        #  In case  the data has a large number of of the same (quantized) 
        #  intensity, expand the range until both limiting values differ from the 
        #  central value by at least 0.25 times the read noise.

        if readnoise:
            L = round(center-0.25)
            M = round(center+0.25)
            R = 0.25*readnoise
            while ((j > 0) and (k < nsky-1) and \
                       ( ((sky[L] - sky[j]) < R) or ((sky[k] - sky[M]) < R))):
                j -= 1
                k += 1

        skymed = np.sum(sky[j:k+1])/(k-j+1)

        #  If the mean is less than the median, then the problem of contamination
        #  is slight, and the mean is what we really want.

        if skymed < skymn : 
            dmod = 3.*skymed-2.*skymn-skymod 
        else: dmod = skymn - skymod

        # prevent oscillations by clamping down if sky adjustments are changing sign
        if dmod*old < 0: clamp = 0.5*clamp
        skymod = skymod + clamp*dmod 
        old = dmod     

#   if redo then goto, START_LOOP

#       
    skew = float( (skymn-skymod)/max([1.,sigma]) )
    nsky = maximm - minimm 

    if debug:
        print('% MMM: Number of unrejected sky elements: ', str(nsky,2), \
                  '    Number of iterations: ',  str(niter))
        print('% MMM: Mode, Sigma, Skew of sky vector:', skymod, sigma, skew   )

    return(skymod,sigma,skew)



import pyfits as py
import numpy as np
import os as os
import glob as glob
import drizzle_position as dp
import acs_determine_focus as adf
import acs_3dpsf as acs_3dpsf
import idlsave as idlsave
import rotate_moments as rm
import ipdb as pdb
import star_galaxy_separation as sgs
import copy as cp
import directories

def psf_cor(    mom_file,
                outfile,
                drizzle_file,
                wavelength,
                mult=1, min_rad=1.5, chip=1,
                constantpsf=0, mscale=0, 
                num_exposures=1, order=3,
                n_chip=2):
    '''
    ;
    ; NAME:                  rrg_psf_cor
    ;
    ; PURPOSE:
    ;    Uses the RRG method to correct moments for PSF effects.  
    ;  corrects using a 3rd order polynomial fit to the moments
    ;the PSF moments should be weighted, this programs corrects for the weighting of the PSF moments
    ;min_rad is the size of the weighting function used to measure the PSF
    ;
    ; INPUTS:
    ;   mom_file- moment file IDL structure produced by rrg_measure_mom
    ;   sex_catalog-SExtractor catalog in fits format
    ;   outfile- output file to store IDL array of variables
    ;   mult multiplier used to find gaussian width from area   
    ;   min_rad- stellar radius
    ;   coeff_file text file with the coefficients of the PSF correction
    ;OUTPUTS:
    ; stores moments and other info in the output file as an idl structure  
    ;
    ; OUTPUT KEYWORD PARAMETERS:
    ; NUM_EXPOSURES : THE NUMBER OF EXPOSURES FOR EACH GALAXY
    
    ; MODIFICATION HISTORY:
    ;   
    ;    April 2003 jrhodes
    ;  May 2005 jrhodes added mscale option, only works with constantpsf
    ;  July 2011 dharvey modified to allow for rotated images,(see section
    ;  for more details)
    
    ; TO DO :    ;CHECK THAT ANGLES ARE CORRECT  PLEASE
    
    ;mom_file has following structure
    
    ;moms={x:x,y:y,xx:xx,yy:yy,xy:xy,xxxx:xxxx,xxxy:xxxy,$
    ;xxyy:xxyy,xyyy:xyyy,yyyy:yyyy,radius:radius,sn:sn,back:back,$
    ;class:class,area:area,area_ab:area_ab,flags:flags,$
    ;a:a,b:b,theta:theta,mag:mag,prob:prob}
    '''
    dirs = directories.return_dirs( )

    moms = py.open(mom_file)[1].data
    uncorrected_xx = moms.xx
    uncorrected_yy = moms.yy
    radius = np.sqrt( ( moms.xx + moms.yy)/2.)
    nGalaxies=len(moms.x)
    

    sigma =  cp.copy(moms.radius)
   
    sigma[ sigma < min_rad ] = min_rad
    

    w2=(1./(sigma*sigma));
    w4=(1./(2.*sigma*sigma*sigma*sigma))



    #; Evaluate moments of PSF
    #;I will work out the focus myself! muwagahaha
    #;Before i get the psf moments i need to get the scat catalogue, which
    #;is found by running tintim_make_scat.pro (for tiny_tim models)
    print 'Getting the psf models from tinytim, cheers Tim!'
    #need to think about this
    #tinytim_make_scat, data_dir=dirs.model_dir, wavelength=filter[0], scat=scat

    scat = idlsave.read( dirs.psf_model_dir+'/TinyTim'+wavelength+'.scat' )['scat']

    #so this function interpolates.
 
    '''
    ;Okay so the way i am going to get the correct psf for each position
    ;in the drizzled image is.
    ; 1. Create a grid of positions which covers the entire field of view
    ;of the drizzles image
    ;2. For each position in the drizzled image, work out how many images
    ;cover this position
    ;3. For each of the images covering this position, work out the psf
    ;4. Then take the average of the moments for each 
    '''

    images = glob.glob( dirs.data_dir+'/j*_drz_sci.fits')
    if len(images) == 0:
        raise ValueError('Cant find single exposures of field')

    nImages = len(images)


    #Now get the positions in the drizzle frame of ref in the individual
    #frame of ref
    
    moms = \
      dp.drizzle_position( drizzle_file, images, moms, dataDir=dirs.data_dir)

    

    #Also get the Orientations in terms of the drizzled image, not
    #WCS


    #Now loop through each position in the psf grid, 
    #and check if the positionsreffram(i,3,*) is non zero! and interpolate
    #to that x and y and then average moms
    #check that the position in the grid is in, then loop through each
    #image, interpolating the psf to this point, 

    #PsfMoms is an vector of many classes of moments
    psf_moms = moments( moms.x, moms.y, nGalaxies )

    FocusArray = np.zeros(nImages)


    for iImage in xrange(nImages):
   
        #Which positions are in the cluster frame
        
        inFrame = moms[images[iImage]+'_INFRAME'] == 1

        #before i determine the psf moments i need to get the focus
        #position of the image in question

        #So get the focus position by fitting the true image stars to the
        #model
        
        focus = adf.acs_determine_focus(  images[iImage], moms, \
                                              drizzle_file, wavelength)

        #Just keep track of the focii i have used through out
        FocusArray[iImage] = focus

        #For all the points in the main drizzled field that are within
        #iImage, interpolate the psf from the ref fram of the single
        #image  to the X,Y of the drizzled image
        
       
        iPsfMoms=acs_3dpsf.acs_3dpsf( moms[images[iImage]+'_X_IMAGE'][inFrame], 
                                    moms[images[iImage]+'_Y_IMAGE'][inFrame],
                                    np.zeros(len(moms[images[iImage]+'_INFRAME'][inFrame]))+focus, \
                                    radius, scat, degree=[3,2,2] )
        #now rotate the moments according to the angle in orient
        iPsfMomsRot = rm.rotate_moments( iPsfMoms, moms[images[iImage]+'_ORIENTAT'][inFrame])
        
        #CHECK THAT ANGLES ARE CORRECT HERE PLEASE
        
        mom_names = iPsfMoms.keys()
        for iMom in mom_names:
            
            #I need to now rotate each moment according to the axis orient 
            #with the drizzled image before stacking.
            if (iMom != 'degree') & \
                (iMom != 'nExposures') &\
                (iMom != 'radius') &\
                (iMom != 'x') & (iMom != 'y'):
                psf_moms[iMom][inFrame] += iPsfMomsRot[iMom]
   
        #then keep count how many images per position
        psf_moms['nExposures'][ inFrame] += 1

        
        #then give the position the value of the averaged psf_moms.

    #Save the focus array
    focuslist = open(dirs.data_dir+'/FocusArray.txt', "wb")
    
    for i in xrange(nImages):
        focuslist.write( "%10s %3.1f \n" % ((images[i].split('_'))[0][2:], FocusArray[i]))



    #Now take the mean of each moment
    


    for iMom in mom_names:
        if (iMom != 'degree') & \
          (iMom != 'nExposures') & \
          (iMom != 'radius') &\
          (iMom != 'x') & (iMom != 'y'):
            psf_moms[iMom] /= psf_moms['nExposures']
            psf_moms[iMom][psf_moms['nExposures'] == 0] = 0.
        
   
        
    #Refind e1 and e2, assuming we want  <q11>-<q22>/<q11>+<q22> not <e1>
    psf_moms.e1=(psf_moms.xx-psf_moms.yy)/ \
        (psf_moms.xx+psf_moms.yy)
    psf_moms.e2=2.*psf_moms.xy/\
        (psf_moms.xx+psf_moms.yy)
    #plot the psf model
    '''
    ps, DrizzleFile+'/Psf_Model.ps', /carre, /psfont
    plot, psf_moms.x, psf_moms.y, title='PSF MODEL FOR '+cluster, $
    xtitle='X [PIXELS]', ytitle='Y [PIXELS]', /iso, /nodata
    plt_evec, psf_moms.x, psf_moms.y, $
    psf_moms.e1, psf_moms.e2, /e1e2, $
    xscale=1000, yscale=1000
    unps
    '''
    #and continue on as per usual...

    pxxw=psf_moms.xx
    pxyw=psf_moms.xy
    pyyw=psf_moms.yy
    pxxxx=psf_moms.xxxx
    pxxxy=psf_moms.xxxy
    pxxyy=psf_moms.xxyy
    pxyyy=psf_moms.xyyy
    pyyyy=psf_moms.yyyy
    w=min_rad
    
    


    #correction for weighgting

    pxx=(pxxw-(pxxw*pyyw+pxxw*pxxw-pxxxx-pxxyy)/(2.*w**2))
    pyy=(pyyw-(pyyw*pyyw+pyyw*pxxw-pyyyy-pxxyy)/(2.*w**2))
    pxy=(pxyw-(pxyw*pxxw+pxyw*pyyw-pxyyy-pxxxy)/(2.*w**2))
  
    


    trace=pxx+pyy
    a=pxx-trace/2
    b=pxy
    c=pxy
    d=pyy-trace/2
    e=pxxxx
    f=pxxxy
    z=pxxyy
    h=pxyyy
    k=pyyyy




    #So the problem is that i am timesing a which is a 
    # [1,10000] psf_vector, we moms which is a vector [a,b]
    ixxa=a*(1-w2*2*moms.xx+w4*(moms.xxxx-moms.xx*moms.xx));
    ixxb=b*(w4*(moms.xxxy-moms.xx*moms.xy));
    ixxc=c*(-w2*(2*moms.xy)+w4*(moms.xxxy-moms.xx*moms.xy));
    ixxd=d*(w4*(moms.xxyy-moms.xx*moms.yy));




    iyya=a*(w4*(moms.xxyy-moms.xx*moms.yy));
    iyyb=b*(w4*(moms.xyyy-moms.yy*moms.xy));
    iyyc=c*(-w2*(2*moms.xy)+w4*(moms.xyyy-moms.yy*moms.xy));
    iyyd=d*(1-w2*2*moms.yy+w4*(moms.yyyy-moms.yy*moms.yy));
    
    ixya=a*(-w2*moms.xy+w4*(moms.xxxy-moms.xy*moms.xx));
    ixyb=b*(1-w2*moms.xx+w4*(moms.xxyy-moms.xy*moms.xy));
    ixyc=c*(-w2*moms.yy+w4*(moms.xxyy-moms.xy*moms.xy));
    ixyd=d*(-w2*moms.xy+w4*(moms.xyyy-moms.xy*moms.yy));
    
    ixx_corr=moms.xx-ixxa-ixxb-ixxc-ixxd;
    iyy_corr=moms.yy-iyya-iyyb-iyyc-iyyd;
    ixy_corr=moms.xy-ixya-ixyb-ixyc-ixyd;



    shear=np.sqrt(0.5*(a+d+trace))
    
    if constantpsf:
         shear=np.zeros(number)+np.sqrt(0.5*(a(0)+d(0)+trace(0)))


    corrected_moments = moments( moms.x, moms.y, len(moms.x))

    gw=np.sqrt( (shear*shear*sigma*sigma)/(shear*shear+sigma*sigma));
    corrected_moments.xx = ((shear/gw)**4)*(ixx_corr-gw*gw);
    corrected_moments.yy = ((shear/gw)**4)*(iyy_corr-gw*gw);
    corrected_moments.xy = ((shear/gw)**4)*(ixy_corr);
    pxxc=a+trace/2
    pyyc=d+trace/2
    pxyc=b
    


    corrected_moments.xxxx = moms.xxxx-e-6*pxxc*moms.xx+6*pxxc*pxxc;
    corrected_moments.xxxy = moms.xxxy-f-3*(pxyc*moms.xx+pxxc*moms.xy)+6*pxyc*pxxc;
    corrected_moments.xxyy = moms.xxyy-z-pxxc*moms.yy-pyyc*moms.xx-4*pxyc*moms.xy+2*pxxc*pyyc+4*pxyc*pxyc;
    corrected_moments.xyyy = moms.xyyy-h-3*(pxyc*moms.yy+pyyc*moms.xy)+6*pxyc*pyyc;
    corrected_moments.yyyy = moms.yyyy-k-6*pyyc*moms.yy+6*pyyc*pyyc;

    corrected_moments['e1'] = (corrected_moments.xx-corrected_moments.yy)/\
      (corrected_moments.xx+corrected_moments.yy)
    corrected_moments['e2'] = (2*corrected_moments.xy)/\
      (corrected_moments.xx+corrected_moments.yy)
  
    #Those moments that were originally zero and -99 make them again hgere
    for i in corrected_moments.keys():
        if i in moms.columns.names:
            corrected_moments[i][moms[i] == -99] = -99
            corrected_moments[i][moms[i] == 0] = 0
            moms[i] = corrected_moments[i]
            
    

      
    moms['gal_size'] = np.sqrt( (corrected_moments.xx +corrected_moments.yy)/2.)
    newcol = [ py.Column(name='shear', format=shear.dtype, array=shear),
               py.Column(name='nExposures', format=psf_moms.nExposures.dtype, \
                         array=psf_moms.nExposures),
                py.Column('xx_uncorrected', format=moms.xx.dtype, array=uncorrected_xx),
                py.Column('yy_uncorrected', format=moms.yy.dtype, array=uncorrected_yy)]
    
    orig_cols = moms.columns
    new_cols = py.ColDefs(newcol)
    
    hdu = py.BinTableHDU.from_columns(orig_cols + new_cols)
    hdu.writeto( outfile, clobber=True)
    

class moments( dict ):

    def __init__(self, x, y, n_objects):
        self.__dict__['x'] = x
        self.__dict__['y'] = y
        self.__dict__['e1'] = np.zeros(n_objects)
        self.__dict__['e2'] = np.zeros(n_objects)
        self.__dict__['xx'] = np.zeros(n_objects)
        self.__dict__['xy'] = np.zeros(n_objects)
        self.__dict__['yy'] = np.zeros(n_objects)
        self.__dict__['xxxx'] = np.zeros(n_objects)
        self.__dict__['xxxy'] = np.zeros(n_objects)
        self.__dict__['xxyy'] = np.zeros(n_objects)
        self.__dict__['xyyy'] = np.zeros(n_objects)
        self.__dict__['yyyy'] = np.zeros(n_objects)
        self.__dict__['nExposures'] = np.zeros(n_objects)

    def __setitem__(self, key, item): 
        self.__dict__[key] = item
        
    def keys(self):
        return self.__dict__.keys()

    def __getitem__(self, key): 
        return self.__dict__[key]



import numpy as np
import pyfits as py
import ipdb as pdb

def acs_limits( x, y, filename):
    '''
    ;PURPOSE : TO RETURN THE INDEX OF THOSE X AND Y POSITIONS THAT
    ;          ARE WITHIN THE FIELD OF VIEW AND NOT ZERO IN THE SINGLE
    ;          IMAGE FRAM
    
    ;INPUTS :
    ;    X : THE X POSITIONS of the sources in pixel coordinates
    ;    Y : Y POSITIONS of the sources in pixels coordinates
    ;    ASTRO : A STRCUTURE FROM EXTAST INCLUDING FILENAME
    
    ;OUTPUTS : A BOOLEAN OF THE INDEXES OF THE X AND Y POSITIONS THAT ARE IN THE FIELD
    '''
    
    image_obj=py.open(filename)
    imagedata = image_obj[0].data
    header = image_obj[0].header
    
    zeroarr = imagedata == 0

    
    isin = (x > 0) & ( x < header['NAXIS1']) & \
        (y > 0) & ( y < header['NAXIS2'])

    zeroarr = np.arange( len( y[isin] ))[imagedata[ y[isin].astype(int), x[isin].astype(int)] == 0]
    #Create an array the size of the entire field even
    #the positions outside the FOV
    SourceArr=np.ones(len(x))  
    
    #All the positions in question make 1 in the array
    #SourceArr[np.round(y-min(y)).astype(int)-1, np.round(x-min(x)).astype(int)-1] = 1
    SourceArr[ isin ] += 1
    
    isin_index = np.arange(len(y))[isin]
    
    #Deduct one off all the positions that arent in the chip
    SourceArr[ isin_index[zeroarr] ] -= 1
    
    #then all the chip positions that are in the FOV AND CHIp are value 2
   
    return SourceArr == 2, header['ORIENTAT']


     



'''
This script will take the output from rrg and convert 
the shears in the format required to be used in lenstool
(using option 7, and the format a, b, theta)

'''

import pyfits as py
import os as os
#import mask_catalogue as mask_catalogue
import numpy as np
from lensing import lenstool as lt

def rrg_to_lenstool( rrg_catalogue,
                         image_file,
                         output_catalogue=None,
                         lenstool_catalogue=None,
                         mask='mask.reg',
                         reference=None,
                         default_src_redshift=1.0):
    '''
    Take the input fits file that is the rrg_catalogue
    and calcualte the a, b and theta.

    Add these to the file and return the catalogue with added
    tagnames

    Also create a ascii file with the catalogue ready to go
    into lentool

    INPUTS :
    rrg_catalogue : the string of the rrg catalogue 
    KEYWORDS :
    reference : a two scalar vector with the ra and dec of the reference of the lenstool 
    output_catalogue : the name of the fits file which has a_lenstool, b_lenstool, theta_lenstool added to the fields
    lenstool_catalogue : a catalogue in the format x, y, a, b, theta, z, mag
    to be used in lenstool
    default_src_redshift : a scalar  of redshifts that the sources galaxies are at
    '''
    image = py.open(image_file)
    rrgCat = py.open( rrg_catalogue )
    
    if os.path.isfile( mask ):
        MaskedRRGCat = mask_catalogue.mask( rrgCat, mask )
    else:
        print 'WARNING: NO MASK FILE FOUND'
        MaskedRRGCat = rrgCat[1].data

    nGalaxies = len( MaskedRRGCat )

    theta = np.arctan2( MaskedRRGCat.gamma2, MaskedRRGCat.gamma1 )*180./np.pi/2.
    rotang = image[0].header['ORIENTAT']
    theta += rotang
    print image[0].header['ORIENTAT']
    gamma = np.sqrt(  MaskedRRGCat.gamma2**2 + MaskedRRGCat.gamma1**2)
    
    size = np.sqrt(( MaskedRRGCat.xx + MaskedRRGCat.yy)/2.)
    semi_major = size*np.sqrt(1.+gamma)
    semi_minor = size*np.sqrt(1.-gamma)

    if reference is None:
        reference = [ np.mean( MaskedRRGCat.RA ), 
                          np.mean( MaskedRRGCat.DEC ) ]


    if lenstool_catalogue is None:
        lenstool_catalogue = rrg_catalogue.split('.')[0]+'.lenstool'
        
    lenstoolCat = open( lenstool_catalogue, 'wb' )

    ngal = np.arange( nGalaxies )+1
    mag = MaskedRRGCat.MAG_AUTO
   

    redshifts = np.zeros( nGalaxies, float)+default_src_redshift
        
    write_array = np.transpose([ ngal, MaskedRRGCat.ra, MaskedRRGCat.dec, \
                                  semi_major, semi_minor, \
                                  theta, \
                                  redshifts,
                                     mag])
    #Remove nans from the file
    rows = np.arange(write_array.shape[0])[ (np.isnan(np.sum(write_array,axis=1)))]

    write_array = np.delete( write_array, rows, axis=0)
            
        
    
    np.savetxt( lenstool_catalogue, write_array,\
			fmt='%i %f %f %f %f %f %f %f', \
                    header='REFERENCE 3 %0.7f %0.7f' % (reference[0], reference[1]))



import numpy as np
import combination as com
import copy as cp
import ipdb as pdb

def rotate_moments( moms, angle):
    
    '''

    ;PURPOSE : TO TAKE THE MOMENTS OF A PSF AND ROTATE THEM SO THEY
    ;          ARE IN THE FRAME OF REFERENCE OF THE DRIZZLED IMAGE
    
    ;         THIS SHOULD ROTATE ALL MOMENTS AND ELLIPTICITIES
    
    
    ;INPUTS :
    ;          MOMS : The structure with the moments to be rotated
    ;                 - xx, xy, yy
    ;                 - xxxx, xxxy, xxyy, xyyy, yyyy
    ;                 - e1, e2
    ;          ANGLE : Angle for the moments to be rotated through
    
    
    ;OUTPUTS :
    ;         MOMS_rotated : The same structure are moms, except
    ;                        moments now rotated
    
    ;METHOD : It will use the equation (36) found in 

    ;http://www.opticsinfobase.org/DirectPDFAccess/71A5781F-D53F-D862-FC6B61687B808297_57703/josa-70-8-920.pdf?da=1&id=57703&seq=0&mobile=no
    '''
    
    #rotate the ellipticities
    #need to make sure angles are in radians
    angle=angle*np.pi/180.
  
    n_galaxies=len(moms.xx)
    #Rotate the quadropole moments first
    #in all case
    
    #Create a matrix of the various quadropole moments
    # a=[[00,10,20],[01,11,21],[02,12,22]]
    #mu_jk = [[0,0,moms.xx],[0,moms.xy,0],[moms.yy,0,0]]
    #create a n array such that the mu_jk correspond to xx, xy, and yy
    mu_jk = np.zeros((3,3,n_galaxies))
    mu_jk[2,0,:]=moms.xx 
    mu_jk[1,1,:]=moms.xy 
    mu_jk[0,2,:]=moms.yy 
  
  
    mu_jk_rot=np.zeros((3, 3, n_galaxies))
    for j in xrange(3):
        for k in xrange(3):
            for r in xrange(j+1):
                for s in xrange(k+1):
                    #Only do those moments we care about
                    if j+k != 2:
                        continue
                    
                        
                    mu_jk_rot[j,k,:] += (-1.)**(k-s)*\
                        com.combination(j, r)*com.combination(k, s)*\
                        np.cos(angle)**(j-r+s)*\
                        np.sin(angle)**(k+r-s)*\
                        mu_jk[j+k-r-s,r+s,:]
              


    #Create a matrix of the various fourth order moments
    # a=[[00,10,20,30,40],$
    #    [01,11,21,31,41],$
    #    [02,12,22,32,42],$
    #    [03,13,23,33,43],$
    #    [04,14,24,34,44]]
    
    
    #mu_jk = [[0,0,moms.xx],[0,moms.xy,0],[moms.yy,0,0]]
    #create a n array such that the mu_jk correspond to xx, xy, and yy
    mu_jklm = np.zeros((5, 5, n_galaxies))
    mu_jklm[4,0,:] = moms.xxxx
    mu_jklm[3,1,:] = moms.xxxy
    mu_jklm[2,2,:] = moms.xxyy
    mu_jklm[1,3,:] = moms.xyyy
    mu_jklm[0,4,:] = moms.yyyy
    
    
    #then rotate them and put them in a new matrix
    mu_jklm_rot=np.zeros((5, 5, n_galaxies))
    for j in xrange(5):
        for k in xrange(5):
            for r in xrange(j+1):
                for s in xrange(k+1):
                    
 
                    if j+k != 4:
                        continue

                    mu_jklm_rot[j,k,:] += (-1)**(k-s)*\
                        com.combination(j, r)*com.combination(k, s)* \
                        np.cos(angle)**(j-r+s)*\
                        np.sin(angle)**(k+r-s)*\
                        mu_jklm[j+k-r-s,r+s, :]
              


    #e1 =(xx-yy)/(xx+yy) e2=2*xy/(xx+yy)
    e1_rotated= (mu_jk_rot[2,0,:]-mu_jk_rot[0,2,:])/\
              (mu_jk_rot[2,0,:]+mu_jk_rot[0,2,:])

    e2_rotated= (2.0*mu_jk_rot[1,1,:])/\
        (mu_jk_rot[2,0,:]+mu_jk_rot[0,2,:])

    e1_rotated[ (mu_jk_rot[2,0,:]+mu_jk_rot[0,2,:]) == 0] = 0
    e2_rotated[ (mu_jk_rot[2,0,:]+mu_jk_rot[0,2,:]) == 0] = 0
    
    MOMS_rotated= cp.deepcopy( moms )

    MOMS_rotated.xx = mu_jk_rot[2,0, :]
    MOMS_rotated.xy = mu_jk_rot[1,1, :]
    MOMS_rotated.yy = mu_jk_rot[0,2, :]
    MOMS_rotated.xxxx = mu_jklm_rot[4,0,:]
    MOMS_rotated.xxxy = mu_jklm_rot[3,1, :]
    MOMS_rotated.xxyy = mu_jklm_rot[2,2, :]
    MOMS_rotated.xyyy = mu_jklm_rot[1,3, :]
    MOMS_rotated.yyyy = mu_jklm_rot[0,4, :]
    MOMS_rotated.e1 = e1_rotated
    MOMS_rotated.e2 = e2_rotated
    MOMS_rotated.radius = moms.radius
    MOMS_rotated.degree = moms.degree
               

    return MOMS_rotated


  




from matplotlib import pyplot as plt
import os as os
import pickle as pkl
from matplotlib import pyplot as plt
from matplotlib import gridspec as gridspec
import sys as sys
import numpy as np
import ipdb as pdb
def star_galaxy_separation( sources, restore=False,
                            savefile=None,
                            NonStop=False,
                            axes=None):

    '''
    ;PURPORSE : To separate out the stars and galaxies in the input
    ;           catalogue
    
    ;INPUTS : sources : a sextractor catalogue of sources
    
    ;KEYWORDS : 
    ; MMM : If selected use the moments from
    ;                 rrg_measure_moms.pro structure
    ; stars : return the indices of the stars
    ; restore : restore a saved file of classes
    ; savefile : save the file to the given string
    
    ;RETURNS : A VECTOR OF THE GALAXIES
    
    ; UPDATES : I Shouldnt be working on the positions of
    ;           or the separation using the single images
    ;           I should be using the stacked and therefore dont
    ;           need the single keyword anymore
    
    ;           Use the moments from Measyrubg moments
    ;             and the radius size for stars use mu max for gals
    '''

    #There will be three plots
    if not restore:
        if axes is None :
            fig = plt.figure( figsize=(10,10))
            gs = gridspec.GridSpec( 2, 2)
            ax1 = plt.subplot( gs[0,0])
            ax2 = plt.subplot( gs[1,0])
            ax3 = plt.subplot( gs[0,1])
        else:
            ax1, ax2, ax3 = axes
            ax1.clear()
            ax2.clear()
            ax3.clear()
        
    radius = np.sqrt(sources['ISOAREA_IMAGE']/np.pi)
    
    #Create a box that separateds the galaxies fom teh stars
    #Each box is define by
    #The line separating Stars and Galaxies
    
  
    nObjects = len(sources['X_IMAGE'])
    object_indexes = np.arange( nObjects )
  
     
    if not os.path.isfile( savefile ):
        gal_star = galStar( savefile, set_defaults=True)
        print 'Loading defaults '+savefile
    else:
        print 'Loading '+savefile
        gal_star = galStar( savefile, set_defaults=False)   
   
  

     
    #Original number ; 1, -3.3, -9, 1, -3.9, -3
   
    galaxies = object_indexes[ (sources['MU_MAX'] > gal_star.GradGal*sources['MAG_AUTO']  + \
                                gal_star.IntGal) &
                                ( sources['MU_MAX'] > gal_star.GalLowCut) & \
                                ( sources['gal_size'] > gal_star.GradStars*sources['MAG_AUTO'] +\
                                gal_star.IntStars )]
                            
    n_galaxies = len(galaxies)
 
    
    #this is just a box aroun dhe stars in the mag vs radius for the 
    #the drizzled image
    stars = object_indexes[ (sources['MU_MAX'] < sources['MAG_AUTO']*gal_star.GradGal + gal_star.IntGal) & \
                             (sources['MU_MAX'] > sources['MAG_AUTO']*gal_star.GradStarsLowCut + gal_star.IntStarsLowCut) & \
                             (sources['gal_size'] < sources['MAG_AUTO']*gal_star.GradStars +  gal_star.IntStars) &\
                            (sources['gal_size'] > gal_star.StarsLowCut )]
    
    n_stars = len(stars)

    if restore:
        return  galaxies, stars
    
    if savefile is not None:

        print 'There are '+str(n_stars)+ ' in the field now plotting'
        #Plot the stars and galaxies to make sure i am taking the right ones
        
        ax1.plot( sources['MAG_AUTO'], sources['gal_size'], 'k,')
        ax1.set_xlabel("Magnitude")
        ax1.set_ylabel("Size")
       

        
        ax1.plot( sources['MAG_AUTO'], gal_star.GradStars*\
                    sources['MAG_AUTO'] +gal_star.IntStars, 'g-' )

        ax1.plot( sources['MAG_AUTO'], np.zeros(nObjects)+gal_star.StarsUpCut,'g-')
         
        ax1.plot( sources['MAG_AUTO'], np.zeros(nObjects)+gal_star.StarsLowCut, 'y-')
     
        ax1.plot( sources['MAG_AUTO'][galaxies], sources['gal_size'][galaxies], 'rD', label='Galaxies')
        ax1.plot( sources['MAG_AUTO'][stars], sources['gal_size'][stars],'y*', label='Stars')
        
        ax1.set_xlim([10,35])
        ax1.set_ylim([0,25])
      
        ax1.legend()
        
    
     
        ax2.plot( sources['MAG_AUTO'], sources['RADIUS'],'k,')

        ax2.plot( sources['MAG_AUTO'][galaxies], sources['RADIUS'][galaxies], 'rD')
        ax2.plot( sources['MAG_AUTO'] [stars], sources['RADIUS'][stars], 'y*')

        ax2.set_xlabel("Magnitude")
        ax2.set_ylabel("Radius")
        ax2.set_xlim([10,30])
        ax2.set_ylim([0,25])
     
     
        ax3.plot( sources['MAG_AUTO'] ,sources['MU_MAX'], 'k,')




        ax3.plot( sources['MAG_AUTO'], np.zeros(nObjects)+gal_star.GalLowCut, 'r-')
        ax3.plot( sources['MAG_AUTO'], gal_star.GradGal*\
                    sources['MAG_AUTO'] + gal_star.IntGal, 'g-')
                    
        ax3.plot( sources['MAG_AUTO'], sources['MAG_AUTO']*\
                gal_star.GradStarsLowCut + gal_star.IntStarsLowCut, 'r-')
        
        
        ax3.plot( sources['MAG_AUTO'][galaxies],sources['MU_MAX'][galaxies],'rD')
        ax3.plot( sources['MAG_AUTO'][stars],sources['MU_MAX'][stars], 'y*')
        ax3.set_xlabel("Magnitude")
        ax3.set_ylabel("Mu Max")
        ax3.set_xlim([10,30])
        ax3.set_ylim([10,30])

        plt.savefig( 'StarGalaxy.pdf' )
        if axes is None:
            plt.show( block=False)
        else:
            plt.draw()
        
       
     

        if NonStop:
            overwrite='c'
        else:
            overwrite = \
                raw_input('Save file exists, overwrite Yes (y) Continue (c) No (n)?')
                  
        
        if overwrite == 'y':
            print 'Writing over gal file and removing j*uncor.cat'
            os.system('rm -fr j*uncor.cat *_cor.cat')
            gal_star.write( savefile )

        else:
            if overwrite == 'c':
                cont=1
                plt.close()
                return galaxies, stars
            else:
                print 'Reseparating'
                galaxies, stars = star_galaxy_separation( sources, savefile=savefile, axes=[ax1,ax2,ax3] )

    plt.close()
    return galaxies, stars

    


class galStar():

        def __init__( self, filename, set_defaults=True ):
            if set_defaults:
                self.defaults( filename )
            else:
                self.load( filename )


        def defaults( self, filename):
            #Use default parameters (first guess)
            #galaxy locuss parameters
            self.GalLowCut = 16
            self.GradGal = 1
            self.IntGal = -4
    
            #Star locuss Parameters
            self.GradStars = -1.1
            self.IntStars = 28.5
        
            self.GradStarsLowCut = 1.
            self.IntStarsLowCut = -5.
    
            self.StarsUpCut = 2.5
            self.StarsLowCut = 2.

            self.write( filename )
            
        def write(self, filename):
            galstar_file = open( filename, 'wb')
            galstar_file.write('#Galaxy Parameters\n')
            galstar_file.write('GalLowCut %5.2f\n' % self.GalLowCut)
            galstar_file.write('GradGal %5.2f\n' % self.GradGal)
            galstar_file.write('IntGal %5.2f\n' % self.IntGal)
            galstar_file.write('#Star Parameters\n')
            galstar_file.write('GradStars %5.2f\n' % self.GradStars)
            galstar_file.write('IntStars %5.2f\n' % self.IntStars)
            galstar_file.write('GradStarsLowCut %5.2f\n' % self.GradStarsLowCut)
            galstar_file.write('IntStarsLowCut %5.2f\n' % self.IntStarsLowCut)
            galstar_file.write('StarsUpCut %5.2f\n' % self.StarsUpCut)
            galstar_file.write('StarsLowCut %5.2f\n' % self.StarsLowCut)

        def load( self, filename ):
            name, values = np.loadtxt( filename, unpack=True, dtype=[('name', object), ('value', float) ])
            
            self.GalLowCut = values[ name  == 'GalLowCut']
            self.GradGal = values[name == 'GradGal']
            self.IntGal = values[ name == 'IntGal']
    
            #Star locuss Parameters
            self.GradStars = values[name == 'GradStars']
            self.IntStars = values[name == 'IntStars']
            
            self.GradStarsLowCut = values[name == 'GradStarsLowCut']
            self.IntStarsLowCut = values[name == 'IntStarsLowCut']
    
            self.StarsUpCut = values[ name == 'StarsUpCut']
            self.StarsLowCut = values[ name == 'StarsLowCut']





import star_galaxy_separation as sgs
import numpy as np
import measure_moms as mm
import acs_model_e as acs_model
import ipdb as pdb
import drizzle_position as dp
import os as os
import pyfits as py
import directories
def acs_determine_focus_metric( true, model ):
    '''
    SO A is the flag to check if the quadrupole moments
    match
    '''

    dof = np.max( [np.float((len(model.e1)-1)), 1])
    goodness_of_fit = np.sum( (true['e1']-model.e1)**2+(true['e2']-model.e2)**2) /dof
    
    return goodness_of_fit


# **************************************************************************

def acs_determine_focus( unknown_focus_image,
                         observed_moments,
                         drizzle_file,
                         wavelength,
                         data_dir=None,
                         psf_model_dir=None,
                         pixel_scale=0.03,
                         r_match=600):
    '''
    ;+      
    ; NAME:
    ;      ACS_DETERMINE_FOCUS
    ;
    ; CATEGORY:
    ;      Reduction of ACS COSMOS data.
    ;
    ; PURPOSE:
    ;      Decides the actual focus value of HST during observations, due to 
    ;      thermal fluctuations that change the distance between the primary
    ;      and secondary mirrors. The offset from the nominal focus position
    ;      is returned, in microns.
    ;
    ; INPUTS:
    ;      moms : Meaured catalogue of moments from the image
    ;
    ; OPTIONAL INPUTS:
    ;      CATALOGUES - Structure containing model TinyTim PSF catalogues. Can
    ;                   be set for speed. If not set, ;read from disk on first run.
    ;
    ; OUTPUTS:
    ;      Focus
    ;
    ; KEYWORDS:
    ;      SINGLE : If set work out the focus for a single exposure
    ;               this should be set to the name of the single exp
    ;
    ; EXAMPLE USE:
    ;      acs_determine_focus, cluster, filter, results
    ;
    ; MODIFICATION HISTORY:
    ;      Feb 05 - All shape moments needed by RRG used, by RM.
    ;      Jan 05 - Written by Richard Massey.
    ;      Aug 13 - Changed by DRH for single exposure
    ;      Sep 19 - Dont use the positions from single image
    ;-
    '''

 
    dirs = directories.return_dirs()

    
    #I need to get the positions of just the stars
    galaxies, stars = sgs.star_galaxy_separation( observed_moments, \
                                                      restore=True,\
                                                    savefile=dirs.data_dir+'/galStar.locus')
    
    #Filter the stars out
    observed_moments_stars = observed_moments[ stars ]
    
    
    #Now I need measure the moments of the stars in the individual image
    #But the x and y here are in the frame of the drizzled frame
    #So need to change these to position in the new fram
    
   
    inframe_stars = observed_moments_stars[observed_moments_stars[unknown_focus_image+'_INFRAME'] == 1]
    

    if not os.path.isfile( unknown_focus_image[:-5]+'_uncor.cat'):
        mm.measure_moms( unknown_focus_image,  'NOCAT_NEED',
                    unknown_focus_image[:-5]+'_uncor.cat',
                    object_catalogue=inframe_stars,
                    xGal=inframe_stars[unknown_focus_image+'_X_IMAGE'],
                    yGal=inframe_stars[unknown_focus_image+'_Y_IMAGE'],
                    mult=2, min_rad=6)
        
    star_moments = py.open( unknown_focus_image[:-5]+'_uncor.cat' )[1].data
    
    #Now get an array of moments for all the possible focus positions
    model_e, focus = acs_model.acs_model_e(star_moments[unknown_focus_image+'_X_IMAGE'],
                                           star_moments[unknown_focus_image+'_Y_IMAGE'], 
                                           wavelength=wavelength )

    #Number of focus positions
    n_focus, nobjects = model_e.xx.shape
    focus = np.arange(16)-10
    average_distance = np.sum(model_e.offset_model, axis=0)/n_focus

    
    # Select only those stars with a suitably close model (since we're not interpolating)
    close_match = np.arange(nobjects)[average_distance < r_match]
    
    n_stars=len(close_match)
    if n_stars < 2:
        print "No stars found in field "
        global_focus=0.
        global_focus_error=100.
        focus = raw_input("No stars are found so please either input focus or cancel : ")
    
    # Tabulate model ellipticities
    model_e1=model_e.e1[:,close_match]
    model_e2=model_e.e2[:,close_match]

    # Tabulate measured ellipticities
    true = star_moments[close_match]
        
     
  #Find best-fit focus
  

    best_fit_focus_indiv = np.zeros( (2, n_stars ))

    for i in xrange(len(close_match)):
        chisq=np.zeros(n_focus) #Absolute chi squared
        for f in xrange(n_focus):
        #Tabulate model ellipticities
            model = moments( 1 )
            model['e1'][0] = model_e.e1[f,close_match][i]
            model['e2'][0] = model_e.e2[f,close_match][i]
            model['x'][0] = model_e.x[close_match][i]
            model['y'][0] = model_e.y[close_match][i]
            model['xx'][0] = model_e.xx[f,close_match][i]
            model['xy'][0] = model_e.xy[f,close_match][i]
            model['yy'][0] = model_e.yy[f,close_match][i]
            model['xxxx'][0] = model_e.xxxx[f,close_match][i]
            model['xxxy'][0] = model_e.xxxy[f,close_match][i]
            model['xxyy'][0] = model_e.xxyy[f,close_match][i]
            model['xyyy'][0] = model_e.xyyy[f,close_match][i]
            model['yyyy'][0] = model_e.yyyy[f,close_match][i]
            
            chisq[f]=acs_determine_focus_metric(true[i], model)
            

        best_fit_focus_indiv[ 0, i ] = focus[ np.argmin( chisq )]
        best_fit_focus_indiv[ 1, i ] = np.min(chisq)

    if n_stars < 5:
        new_best_fit_focus =  best_fit_focus_indiv[0, np.argmin(best_fit_focus_indiv[1,:])]
        #Just for the purposes of the test! MAKE SURE I DELTE

    else:
        best_fit_focus = np.median( best_fit_focus_indiv[0,:] )
        #If the median is between two take the best fit out of those two
        if np.round( best_fit_focus ) != best_fit_focus:
            floor_chi = best_fit_focus_indiv[1, best_fit_focus_indiv[0,:] ==  \
                                                     np.floor( best_fit_focus )]
            ceil_chi = best_fit_focus_indiv[1, best_fit_focus_indiv[0,:] ==  \
                                                     np.ceil( best_fit_focus )]
                                                     
            if (len(ceil_chi) == 0) | (len(floor_chi) == 0):
                chi_weighted_mean = np.sum( best_fit_focus_indiv[0,:]/best_fit_focus_indiv[1,:])/\
                  np.sum(1./best_fit_focus_indiv[1,:])
                
                new_best_fit_focus = np.round(chi_weighted_mean)
             
            else:
                min_floor_chi = np.min( floor_chi )
                min_ceil_chi = np.min( ceil_chi )
                focii = np.append( np.floor( best_fit_focus ), np.ceil( best_fit_focus ))
                chi = np.append( min_floor_chi, min_ceil_chi )
                new_best_fit_focus = focii[ np.argmin( chi ) ]
        else:
            
            return best_fit_focus
                                                                                                
                                                                          
    #Store the best one
    
    return new_best_fit_focus


class moments( dict ):

    def __init__( self, n_objects ):

        self.__dict__['x'] = np.zeros(n_objects)
        self.__dict__['y'] = np.zeros(n_objects)
        self.__dict__['e1'] = np.zeros(n_objects)
        self.__dict__['e2'] = np.zeros(n_objects)
        self.__dict__['xx'] = np.zeros(n_objects)
        self.__dict__['xy'] = np.zeros(n_objects)
        self.__dict__['yy'] = np.zeros(n_objects)
        self.__dict__['xxxx'] = np.zeros(n_objects)
        self.__dict__['xxxy'] = np.zeros(n_objects)
        self.__dict__['xxyy'] = np.zeros(n_objects)
        self.__dict__['xyyy'] = np.zeros(n_objects)
        self.__dict__['yyyy'] = np.zeros(n_objects)



    def keys(self):
        return self.__dict__.keys()

    def __getitem__(self, key): 
        return self.__dict__[key]
